#include <iostream>
#include <sstream>
#include <cstring>
#include <algorithm>
#include <sys/types.h>
#include <unistd.h>
#include "exception.h"

std::deque<char *> DAQpp::Exception::messages;

const char *DAQpp::Exception::new_message(const std::string &s) const
{
    char *msg;
    if (messages.size()>50)
    {
        msg = messages.front();
        messages.pop_front();
        delete [] msg;
    }
    msg = new char [ s.length() ];
    strcpy(msg, s.c_str());
    messages.push_back( msg );

    return msg;
}


DAQpp::Exception::Exception()
{}

DAQpp::Exception::Exception(const DAQpp::Exception &e)
    : _why(e._why)
{
}

DAQpp::Exception::~Exception() throw()
{}

const char* DAQpp::Exception::what() const throw()
{
    std::ostringstream ostr;
    ostr << "process (" << getpid() << "): ";
    if (_why.empty() )
        ostr << "DAQpp::Exception";
    else
        ostr << _why;

    return new_message(ostr.str());
}

DAQpp::ParameterException::ParameterException(const std::string &pname):
    name(pname)
{}

DAQpp::ParameterException::ParameterException(const DAQpp::ParameterException &e):
    DAQpp::Exception(e)
{
    name = e.name;
}

const char * DAQpp::ParameterException::what() const throw()
{
    std::ostringstream ostr;
    ostr << "process (" << getpid() << ") "
         << ": " << _why;

    return new_message(ostr.str());
}

DAQpp::ParameterException::~ParameterException() throw()
{
}

DAQpp::ParameterNonExistent::ParameterNonExistent(const std::string &name)
: ParameterException(name)
{
    std::ostringstream ostr;
    ostr << "Parameter " << get_name() << " does not exist";
    _why = ostr.str();
}
DAQpp::ParameterTypeDoesNotMatch::ParameterTypeDoesNotMatch(const std::string &name)
: ParameterException(name)
{
    std::ostringstream ostr;
    ostr << "Parameter type of "
         << get_name()
         << " does not match"
    ;
    _why = ostr.str();
}

DAQpp::NoMemoryException::NoMemoryException(const std::string &s)
{
    _why = s;
}

DAQpp::IOException::IOException(const std::string &s)
{
    _why = s;
}

std::ostream &operator<<( std::ostream &os, const DAQpp::Exception &ex)
{
    os << ex.what();
    return os;
}
