/*
 * DAQppSOAP.cc
 *
 *  Created on: 9 Jan 2011
 *      Author: lacasta
 */

#include <DAQ++/Thread.h>
#include "daqppService.h"
#include "DAQppSOAP.h"

class DAQpp_server : public DAQpp::Thread
{
    private:
        int port;
        int _error;
        daqppService server;

    public:
        DAQpp_server(int p) : port(p), _error(0) {
            ((soap *)&server)->bind_flags |= SO_REUSEADDR;
        }
        ~DAQpp_server()
        {
            close_socket();
        }
        void run()
        {
            _error = server.run(port);
        }
        void close_socket()
        {
            server.soap_close_socket();
        }
        int error()
        {
            return _error;
        }

        soap *get_soap() { return (soap *)&server; }
};

struct DAQpp::DAQppSOAP_priv
{
        DAQpp_server server;
        DAQppSOAP_priv(int port) : server(port) {}
};

DAQpp::DAQppSOAP::DAQppSOAP(int port)
{
    _priv = new DAQppSOAP_priv(port);
}

DAQpp::DAQppSOAP::~DAQppSOAP()
{
    delete _priv;
}

void DAQpp::DAQppSOAP::serve(bool threaded)
{
    if (threaded)
        _priv->server.start();
    else
        _priv->server.run();
}

int DAQpp::DAQppSOAP::error()
{
    return _priv->server.error();
}

std::string DAQpp::DAQppSOAP::strerror() const
{
   return std::string( *soap_faultstring(_priv->server.get_soap()) );
}

void DAQpp::DAQppSOAP::stop_serving()
{
    _priv->server.close_socket();
}

bool DAQpp::DAQppSOAP::is_running() const
{
    return _priv->server.is_running();
}
