/*
 * MsgManager.cc
 *
 *  Created on: 13 Jan 2011
 *      Author: lacasta
 */
#include <DAQ++/logger.h>
#include <DAQ++/DAQmanager.h>
#include "MsgManager.h"

DAQpp::MsgManager::MsgManager()
{
    DAQpp::DAQmanager *mngr = DAQpp::DAQmanager::theDAQmanager();
    mngr->addObserver( *this );

    /*
     * Detect existing objects
     */
    const DAQpp::ModulesMap &mmap = mngr->ModuleMap();
    DAQpp::ModulesMap::const_iterator ip;
    for (ip = mmap.begin(); ip != mmap.end(); ++ip)
    {
        std::string name = std::string("mod:") + ip->first;
        ip->second->addObserver( parameter );
        daq_objects[name] = ip->second;
    }
    DAQpp::RunManagerMap::const_iterator ir;
    const DAQpp::RunManagerMap &rmap = mngr->RunMmap();
    for (ir = rmap.begin(); ir != rmap.end(); ++ir)
    {
        std::string name = std::string("rm:") + ir->first;
        ir->second->addObserver( parameter );
        ir->second->get_status().addObserver( state );
        daq_objects[name] = ir->second;
    }
}

/**
 * This routine receives the notifications of DAQmanager
 */
void DAQpp::MsgManager::update(Observable *o, Argument *arg)
{
    Arg< DAQpp::DAQmanager::newObjArg> &msg =
        *static_cast< Arg< DAQpp::DAQmanager::newObjArg> * > (arg);
        DAQpp::Module *module;
        DAQpp::RunManager *run_manager;

        switch (msg().first)
        {
            case DAQpp::DAQmanager::nwModule:
                module = reinterpret_cast<DAQpp::Module *>(msg().second);
                module->addObserver( theMsgManager()->parameter );
                break;
            case DAQpp::DAQmanager::rmModule:
                module = reinterpret_cast<DAQpp::Module *>(msg().second);
                daq_objects.erase( std::string("mod:") + module->get_id()());
                break;
            case DAQpp::DAQmanager::nwRunManager:
                run_manager = reinterpret_cast<DAQpp::RunManager *>(msg().second);
                run_manager->get_status().addObserver( theMsgManager()->state );
                run_manager->addObserver( theMsgManager()->parameter );
                break;
            case DAQpp::DAQmanager::rmRunManager:
                run_manager = reinterpret_cast<DAQpp::RunManager *>(msg().second);
                daq_objects.erase( std::string("rm:")+run_manager->get_id()() );
                break;
            default:
                break;
        }
}

/**
 * We are notified of any change of state of a RunManager through
 * this method that transmits the corresponding new state to the
 * RunMenu associated to the "changing" RunManager
 *
 */
void DAQpp::MsgManager::StateChanged::update(Observable *o, Argument *arg)
{
    DAQpp::ostream os;
    //Arg<DAQpp::RunState::StatusType> *st = static_cast< Arg<DAQpp::RunState::StatusType> * > (arg);
    DAQpp::RunState *state = dynamic_cast<DAQpp::RunState *>(o);
    if ( state == 0 )
            return;

    MsgManager *mngr = MsgManager::theMsgManager();
    mngr->acquire();
    mngr->notif_state[state->get_daqid()] = state->get_name();
    //os << DAQpp::loglevel(DAQpp::Log::info) << "SOAP MsgManager adding state [" << state->get_daqid().str() << " : " << state->get_name() << "]" << std::endl;
    mngr->release();

    os << DAQpp::loglevel(DAQpp::Log::debug)
       << state->get_name()
       << " changes state to "
       << state->status_str(state->get_status()) << std::endl;
}

void DAQpp::MsgManager::ParameterChanged::update(Observable *o, Argument *arg)
{
    DAQpp::ostream os;
    DAQpp::Parameter *par = dynamic_cast<DAQpp::Parameter *>(o);

    if ( par == 0 )
    {
        os << DAQpp::loglevel(DAQpp::Log::error)
           << "MsgManager: I got a Request of parameter change from a non-parameter"
           << std::endl;
        return;
    }


    // Update the ParamSliders
    std::string key = par->get_id().str() + "\t" + par->get_name();
    MsgManager *mngr = MsgManager::theMsgManager();
    mngr->acquire();
    mngr->notif_parameters[key] = par->get_name();
    //os << DAQpp::loglevel(DAQpp::Log::info) << "SOAP MsgManager adding parameter [" << key << " : " << par->get_name() << "]" << std::endl;
    mngr->release();

    std::string id = par->get_id().str() + "/" + par->get_name();
    os << DAQpp::loglevel(DAQpp::Log::debug)
       << "Parameter " << id << " changes to " << par->get_as_string() << std::endl;
};

void DAQpp::MsgManager::ObjectAddedRemoved::update(Observable *o, Argument *arg)
{
    // DO nothing for now

};

