/*
 * daqppClient.h
 *
 *  Created on: Dec 15, 2014
 *      Author: lacasta
 */

#ifndef DAQ___SOAP_DAQPPCLIENT_H_
#define DAQ___SOAP_DAQPPCLIENT_H_

#include <iosfwd>
#include <string>
#include <vector>
#include <algorithm>
#include <DAQ++/soap/soapdaqppSOAPProxy.h>
#include <DAQ++/soap/clientParams.h>
#include <DAQ++/Parameter.h>
#include <DAQ++/RunState.h>


class daqppClient
{
    private:
        template<class T>
        static bool destroy(T *ptr)
        {
            delete ptr;
            return true;
        }
    public:
        typedef std::vector< std::string > NameList;
        typedef std::vector< DAQpp::Parameter *> ParameterList;
        typedef std::vector< DAQpp::RunState *> StateList;
        enum DebugOption { debugIN=1, debugOUT=2 };

        template<class T>
        static void empty_list( std::vector<T *> &V )
        {
                std::remove_if(V.begin(), V.end(), destroy<T>);
        }
    private:
        daqppSOAPProxy *server;

    public:
        daqppClient(const char *uri=0);
        virtual ~daqppClient();

        void connect(const char *uri);
        void set_logging(int opt=3);
        int getModuleList(NameList &lst);
        int getRunManagerList(NameList &lst);
        int getRunManagerModules(const std::string &daqid, NameList &lst);
        int getStatus(const std::string &daqid, DAQpp::RunState &status);
        int getParametersList(const std::string &daqid, NameList &lst);

        int GetReady(const std::string &daqid);
        int Pause(const std::string &daqid);
        int Continue(const std::string &daqid);
        int Reset(const std::string &daqid);

        int setMaxEvents(const std::string &daqid, int nevts);
        int StartRunManager(const std::string &daqid);
        int StopRunManager(const std::string &daqid);
        int stopServer();

        /**
         * Monitor data
         * getMonitorData makes all the transactions to get the data
         * from the server and then calls to monitor_data passing an
         * input stream from where the data can be extracted by reading
         */
        int getMonitorData(const std::string &daqid);
        int resetMonitorData(const std::string &daqid);
        virtual void monitor_data(std::istream &idata) {}

        static void wait(double s);

        /**
         * Parameters
         */
        template<typename Type, typename anyType>
        void get_vector_value(const std::string &daqid, const std::string &name, std::vector<Type> &out)
        {
            ns1__getParameterResponse parameter;
            ns1__ParameterID par_id;
            par_id.holderID = daqid;
            par_id.parName = name;
            server->getParameter(&par_id, parameter);
            get_vpar_value<Type, anyType>(parameter.param, out);
        }

        template<typename Type, typename anyType>
        Type get_value(const std::string &daqid, const std::string &name)
        {
            ns1__getParameterResponse parameter;
            ns1__ParameterID par_id;
            par_id.holderID = daqid;
            par_id.parName = name;
            server->getParameter(&par_id, parameter);
            return get_par_value<Type, anyType>(parameter.param);
        }

        template<typename Type, typename anyType>
        int set_value(const std::string &daqid, const std::string &name, Type &val)
        {
            int rc;
            ns1__Parameter *ppar = set_parameter<Type, anyType>(server, name, val);
            ppar->daqid = daqid;
            server->setParameter(ppar, rc);
            return rc;
        }
        template<typename Type, typename anyType>
        int set_vector_value(const std::string &daqid, const std::string &name, std::vector<Type> &val)
        {
            int rc;
            ns1__Parameter *ppar = set_vparameter<Type, anyType>(server, name, val);
            ppar->daqid = daqid;
            server->setParameter(ppar, rc);
            return rc;
        }

        /**
         * Updates.
         * This command retrieves a list of changes in parameters and States since
         * the last time we did such a request. This is used to "update" the
         * information that a client has on the DAQobjects.
         */
        int sendUpdate( ParameterList &parList, StateList &stateList );

};

std::ostream &operator<<(std::ostream &os, const ns1__Status &st);

#endif /* DAQ___SOAP_DAQPPCLIENT_H_ */
