/*
 * daqppService.cc
 *
 *  Created on: 20 Dec 2010
 *      Author: lacasta
 *
 *      Contains the methods needed by soapdaqppSOAPService
 */
#ifdef HAVE_CONFIG
#include <config.h>
#endif

#include <cstdlib>
#include <sstream>
#include <string>
#include <vector>
#include <algorithm>
#include <typeinfo>
#include <cxxabi.h>
#include <DAQ++/DAQmanager.h>
#include <DAQ++/Module.h>
#include <DAQ++/RunManager.h>
#include <DAQ++/logger.h>
#include <DAQ++/ObserverModel.h>
#include "daqppService.h"
#include "daqppSOAP.nsmap"
#include "soapStub.h"
#include "MsgManager.h"

#if defined(DEBUG)
#include <cstdio>
#define PDEBUG(fmt, args...) \
    printf( "[%-15s] " fmt "\n", __FUNCTION__ , ## args)
#else
#define PDEBUG(...)
#endif


#ifdef HAS_PYTHON
#include <Python.h>
#if (PY_VERSION_HEX < 0x02050000)
typedef int Py_ssize_t;
typedef inquiry lenfunc;
typedef intargfunc ssizeargfunc;
typedef intintargfunc ssizessizeargfunc;
typedef intobjargproc ssizeobjargproc;
typedef intintobjargproc ssizessizeobjargproc;
typedef getreadbufferproc readbufferproc;
typedef getwritebufferproc writebufferproc;
typedef getsegcountproc segcountproc;
#endif

#if (PY_VERSION_HEX <= 0x02040400)
typedef char * pycharp_t;
#define Py_DecRef Py_DECREF
#else
typedef const char * pycharp_t;
#endif
PyObject *
load_module(const std::string &module, const std::string &method)
{
    PyObject *pymodule, *pymethod;

    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();

    pymodule = PyImport_ImportModule((char *)module.c_str());
    if (pymodule==0)
    {
        std::cout << "Could not load " << module << std::endl;
        PyGILState_Release(gilstate);
        return 0;
    }
    pymethod = PyObject_GetAttrString(pymodule, (char *)method.c_str());
    if (!pymethod)
    {
        std::cout << "Could not find " << method << " in " << module << std::endl;
        PyGILState_Release(gilstate);
        return 0;
    }

    if ( !PyCallable_Check(pymethod))
    {
        std::cout << method << " in " << module << " is not a callable function" << std::endl;
        PyGILState_Release(gilstate);
        return 0;
    }

    PyGILState_Release(gilstate);
    return pymethod;
}
#endif


int daqppSOAPService::getModuleList(int getModuleListRequest, struct ns1__getModuleListResponse &_param_1) { return SOAP_OK;}
int daqppSOAPService::getRunManagerList(int getRunManagerListRequest, struct ns1__getRunManagerListResponse &_param_2) { return SOAP_OK;}
int daqppSOAPService::getParameterNames(std::string getParameterNamesRequest, struct ns1__getParameterNamesResponse &_param_3) { return SOAP_OK;}
int daqppSOAPService::getStatus(std::string getStatusRequest,struct ns1__getStatusResponse &_param_4) { return SOAP_OK;}
int daqppSOAPService::getParameter(ns1__ParameterID *getParameterRequest, struct ns1__getParameterResponse &_param_5) { return SOAP_OK;}
int daqppSOAPService::setParameter(ns1__Parameter *setParameterRequest, int &setParameterResponse) { return SOAP_OK;}
int daqppSOAPService::activateRunManager(std::string activateRMRequest, int &activateRMResponse) { return SOAP_OK;}
int daqppSOAPService::StartRunManager(std::string StartRunManagerRequest, int &StartRunManagerResponse) { return SOAP_OK;}
int daqppSOAPService::StopRunManager(std::string StopRunManagerRequest, int &StopRunManagerResponse) { return SOAP_OK;}
int daqppSOAPService::GetReady(std::string GetReadyRequest, int &GetReadyResponse) { return SOAP_OK;}
int daqppSOAPService::Pause(std::string PauseRequest, int &PauseResponse) { return SOAP_OK;}
int daqppSOAPService::Continue(std::string ContinueRequest, int &ContinueResponse) { return SOAP_OK;}
int daqppSOAPService::Reset(std::string ResetRequest, int &ResetResponse) { return SOAP_OK;}
int daqppSOAPService::setMaxEvents(ns1__MaxEvents *setMaxEventsRequest, int &setMaxEventsResponse) { return SOAP_OK;}
int daqppSOAPService::getMonitorData(std::string getMonitorDataRequest, xsd__base64Binary &getMonitorDataResponse) { return SOAP_OK;}
int daqppSOAPService::stopServer(int stopServerRequest, int &stopServerResponse) { return SOAP_OK;}
int daqppSOAPService::resetMonitorData(std::string resetMonitorDataRequest, int &resetMonitorDataResponse) { return SOAP_OK;}
int daqppSOAPService::sendUpdate(int sendUpdateRequest, struct ns1__sendUpdateResponse &_param_6) { return SOAP_OK; }
int daqppSOAPService::getRunManagerModules(std::string daqid, struct ns1__getRunManagerModulesResponse &_param_7) { return SOAP_OK; }

#include "daqpp_wsdl.h"

int http_get(struct soap *soap)
{
    char *s = strchr(soap->path, '?');
    if (!s || strcmp(s, "?wsdl"))
        return SOAP_GET_METHOD;

    int nchar = strlen(daqpp_wsdl);
    const char *end_ptr = daqpp_wsdl + nchar -1;
    const char *wsdl = daqpp_wsdl;

    soap->http_content = "text/xml"; // HTTP header with text/xml content
    soap_response(soap, SOAP_FILE);
    for (;;)
    {
        if (wsdl>=end_ptr)
            break;

        size_t r = end_ptr - wsdl;
        if ( r > sizeof(soap->tmpbuf) )
            r = sizeof(soap->tmpbuf);

        memcpy(soap->tmpbuf, wsdl,r);
        wsdl += r;

        if (soap_send_raw(soap, soap->tmpbuf, r))
            break; // can't send, but little we can do about that
    }
    soap_end_send(soap);
    return SOAP_OK;
}


// a few utils
inline std::string trim_right(const std::string &s) {
   std::string b=" \t\n";
   std::string str = s;
   return str.erase( str.find_last_not_of( b ) +1 );
}
inline std::string trim_left(const std::string &s) {
   std::string b=" \t\n";
   std::string str = s;
   return str.erase( 0,str.find_first_not_of( b ) );
}
inline std::string trim_str(const std::string &s) {
   std::string str = s;
   return trim_left( trim_right( str ) );
}

static std::string parse_type_name( const DAQpp::Parameter *par, std::string &name ) {
   const std::string ptype("DAQpp::Par<");
   const int psize=ptype.size();
   const std::type_info  &ti = typeid( *par );
   char *abi_out;

   int status;
   std::string::size_type zp, ip;
   abi_out = abi::__cxa_demangle(ti.name(), 0, 0, &status);
   name = abi_out;
   free(abi_out);
   ip = name.find(ptype);
   if (ip==name.npos)
      return std::string();

   zp = name.rfind(">");
   if (zp==name.npos)
      return std::string();

   std::string type = trim_str( name.substr(ip+psize,zp-ip-psize) );
   // remove the "allocator" part of a vector
   std::string::size_type bra, ket;
   bra = type.find('<');
   if ( bra != type.npos )
   {
       ket = type.find(',', bra);
       if ( ket != type.npos )
       {
           type = type.substr(0, ket);
           type += '>';
       }
   }
   return type;
}

typedef void (*SetterFunction)(DAQpp::Parameter *, xsd__anyType *,bool);
typedef xsd__anyType *(*GetterFunction)(soap *, DAQpp::Parameter *);
struct GetSetfunctions
{
        SetterFunction set;
        GetterFunction get;
};

template<typename anyType>
anyType *instantiate_type(soap *ss)
{
    anyType hack;
    anyType *any = new anyType();
    struct soap_clist *cp = soap_link(ss, NULL, hack.soap_type(), -1, soap_fdelete);
    cp->ptr = (void *)any;
    any->soap = ss;
    return any;
}

#ifdef HAS_PYTHON
void set_python_param(DAQpp::Parameter *P, xsd__anyType *any, bool notify)
{
    PyObject *pickle, *str, *obj, *old, *base64;
    PyGILState_STATE gilstate;

    gilstate = PyGILState_Ensure();
    pickle = load_module("cPickle", "loads");
    base64 = load_module("base64", "b64decode");
    str = PyObject_CallFunction(base64,(char *)"s", any->__item);
    obj = PyObject_CallFunctionObjArgs(pickle, str, 0);

    old = ((DAQpp::Par< PyObject *> *)(P))->get_value();
    Py_DecRef(old);

    ((DAQpp::Par< PyObject *> *)(P))->set_value(obj, notify);
    Py_DecRef(pickle);
    Py_DecRef(str);
    Py_DecRef(base64);
    PyGILState_Release(gilstate);
}

xsd__anyType *get_python_param(soap *ss, DAQpp::Parameter *P)
{
    PyObject *obj, *pickle, *str;
    PyGILState_STATE gilstate;
    char *buffer;
    Py_ssize_t size;
    xsd__base64Binary_ *base64;

    gilstate = PyGILState_Ensure();

    obj    = ((DAQpp::Par< PyObject *> *)(P))->get_value();
    pickle = load_module("cPickle", "dumps");
    str    = PyObject_CallFunctionObjArgs(pickle, obj, 0);
    PyString_AsStringAndSize(str, &buffer, &size);

    base64 = instantiate_type<xsd__base64Binary_>(ss);
    base64->__item.__size = size;
    base64->__item.__ptr = (unsigned char *)soap_malloc(ss, size);
    memcpy(base64->__item.__ptr, buffer, base64->__item.__size);


    Py_DecRef(str);
    Py_DecRef(pickle);

    PyGILState_Release(gilstate);
    return base64;
}

#endif

template<typename Type, typename anyType>
void set_parameter_value(DAQpp::Parameter *P, xsd__anyType *any, bool notify)
{
    Type v = (( anyType *)any)->__item;
    ((DAQpp::Par< Type > *)(P))->set_value(v, notify);
}

template<typename Type, typename anyType>
xsd__anyType *get_parameter_value(soap *ss, DAQpp::Parameter *P)
{
    anyType *any = instantiate_type<anyType>(ss); //new anyType();
    any->__item = ((DAQpp::Par< Type > *)(P))->get_value();
    return any;
}




template<typename Type, typename anyType>
struct inserter
{
        soap *ss;
        inserter(soap *s) : ss(s) {}
        anyType *operator()(Type v)
        {
            anyType *a = instantiate_type<anyType>(ss);
            a->__item = v;
            return a;
        }
};


template<typename Type, typename anyType>
xsd__anyType *get_par_vector(soap *ss, DAQpp::Parameter *P)
{
    ns1__valueType *any = soap_new_ns1__valueType(ss, -1);
    std::vector<Type> &val = ((DAQpp::Par< std::vector<Type> > *)(P))->get_value();
    any->item.resize(val.size(), (xsd__anyType *)0);
    std::transform(val.begin(), val.end(), any->item.begin(), inserter<Type, anyType>(ss));
    return any;
}

template<typename Type, typename anyType>
void set_par_vector(DAQpp::Parameter *P, xsd__anyType *anyValue, bool notify)
{
    ns1__valueType *vt = static_cast<ns1__valueType *>(anyValue);
    std::vector<Type> &val = ((DAQpp::Par< std::vector<Type> > *)(P))->get_value();
    std::vector< xsd__anyType *>::iterator ip;

    val.clear();
    for ( ip = vt->item.begin(); ip!=vt->item.end(); ++ip )
    {
        anyType *any = (anyType *)(*ip);
        val.push_back( any->__item );
    }
    if (notify)
    {
        Arg< std::vector<Type> > arg(val);
        P->notifyObservers( &arg );
    }
}

class DAQppTypeMap
{
    private:
        static void init_map();
    public:
        static std::map< std::string, struct GetSetfunctions > theMap;
    public:
        DAQppTypeMap()
        {
            if (theMap.empty())
                init_map();
        }

};
std::map< std::string, struct GetSetfunctions > DAQppTypeMap::theMap;
static DAQppTypeMap _the_type_map;

void DAQppTypeMap::init_map() {
   struct __S {
     const char *name;
     struct GetSetfunctions F;
   };
   __S _slist[] = {
      {"bool", { set_parameter_value<bool, xsd__boolean>, get_parameter_value<bool, xsd__boolean>} },
      {"char", { set_parameter_value<char, xsd__byte_>, get_parameter_value<char, xsd__byte_>} },
      {"short", { set_parameter_value<short, xsd__short>, get_parameter_value<short, xsd__short>} },
      {"int", { set_parameter_value<int, xsd__int>, get_parameter_value<int, xsd__int>} },
      {"long", { set_parameter_value<long, xsd__long>, get_parameter_value<long, xsd__long>} },
      {"unsigned char", { set_parameter_value<unsigned char, xsd__unsignedByte_>, get_parameter_value<unsigned char, xsd__unsignedByte_>} },
      {"unsigned short", { set_parameter_value<unsigned short, xsd__unsignedShort>, get_parameter_value<unsigned short, xsd__unsignedShort>} },
      {"unsigned int", { set_parameter_value<unsigned int, xsd__unsignedInt>, get_parameter_value<unsigned int, xsd__unsignedInt>} },
      {"unsigned long", { set_parameter_value<unsigned long, xsd__unsignedLong>, get_parameter_value<unsigned long, xsd__unsignedLong>} },
      {"float", { set_parameter_value<float, xsd__float>, get_parameter_value<float, xsd__float>} },
      {"double", { set_parameter_value<double, xsd__double>, get_parameter_value<double, xsd__double>} },
      {"std::string", { set_parameter_value<std::string, xsd__string>, get_parameter_value<std::string, xsd__string>} },
      {"std::vector<char>", { set_par_vector<char, xsd__byte_>, get_par_vector<char, xsd__byte_> }},
      {"std::vector<bool>", { set_par_vector<bool, xsd__boolean>, get_par_vector<bool, xsd__boolean> }},
      {"std::vector<short>", { set_par_vector<short, xsd__short>, get_par_vector<short, xsd__short> }},
      {"std::vector<int>", { set_par_vector<int, xsd__int>, get_par_vector<int, xsd__int> }},
      {"std::vector<long>", { set_par_vector<long, xsd__long>, get_par_vector<long, xsd__long> }},
      {"std::vector<unsigned char>", { set_par_vector<unsigned char, xsd__unsignedByte_>, get_par_vector<unsigned char, xsd__unsignedByte_> }},
      {"std::vector<unsigned short>", { set_par_vector<unsigned short, xsd__unsignedShort>, get_par_vector<unsigned short, xsd__unsignedShort> }},
      {"std::vector<unsigned int>", { set_par_vector<unsigned int, xsd__unsignedInt>, get_par_vector<unsigned int, xsd__unsignedInt> }},
      {"std::vector<unsigned long>", { set_par_vector<unsigned long, xsd__unsignedLong>, get_par_vector<unsigned long, xsd__unsignedLong> }},
      {"std::vector<float>", { set_par_vector<float, xsd__float>, get_par_vector<float, xsd__float> }},
      {"std::vector<double>", { set_par_vector<double, xsd__double>, get_par_vector<double, xsd__double> }},
      {"std::vector<std::string>", { set_par_vector<std::string, xsd__string>, get_par_vector<std::string, xsd__string> }},
#ifdef HAS_PYTHON
      {"_object*", {set_python_param, get_python_param}},
#endif
      {0,{0, 0}}
   };
   int i;
   for (i=0;_slist[i].name;i++)
      theMap[_slist[i].name] = _slist[i].F;

}


xsd__anyType *get_parameter(soap *ss, DAQpp::Parameter *P)
{
    std::map< std::string, GetSetfunctions >::iterator ip;
    std::string name;
    std::string type = parse_type_name(P,name);

    ip = DAQppTypeMap::theMap.find(type);
    if (ip != DAQppTypeMap::theMap.end())
    {
        return ip->second.get(ss, P);
    }
    else
        return 0;
}

int set_parameter(ns1__Parameter *P, DAQpp::Parameter *dpar)
{
    std::map< std::string, GetSetfunctions >::iterator ip;
    std::string name;
    std::string type = parse_type_name(dpar,name);

    ip = DAQppTypeMap::theMap.find(type);
    if (ip != DAQppTypeMap::theMap.end())
    {
        ip->second.set(dpar, static_cast<xsd__anyType *>(P->value->value->item[0]), true);
        return 0;
    }
    else
        return 1;
}

DAQpp::DAQObject *find_object(const std::string &daqid)
{
    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    DAQpp::DAQObject *obj = daqm->find_module(daqid);
    if (!obj)
        obj = daqm->find_runmanager(daqid);

    return obj;
}

daqppService::daqppService()
{
    init_service();
}

daqppService::daqppService(const struct soap &soap)
    : daqppSOAPService(soap)
{
        init_service();
}

daqppService::daqppService(soap_mode iomode)
    : daqppSOAPService(iomode)
{
    init_service();
}

daqppService::daqppService(soap_mode imode, soap_mode omode)
    : daqppSOAPService(imode, omode)
{
    init_service();
}

void daqppService::init_service()
{
    mngr = DAQpp::MsgManager::theMsgManager();
    this->fget=http_get;
}

/// Web service operation 'getModuleList' (returns error code or SOAP_OK)
int daqppService::getModuleList(int dummy, struct ns1__getModuleListResponse &_param_1)
{
    ns1__DAQidList *idlist = static_cast<ns1__DAQidList *>(instantiate_type<ns1__DAQidList>(this));
    _param_1.idlist = idlist;
    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    DAQpp::ModulesMap::const_iterator ip;
    const DAQpp::ModulesMap &mmap = daqm->ModuleMap();
    for (ip = mmap.begin(); ip != mmap.end(); ++ip)
    {
        idlist->name.push_back(ip->first);
    }
    return SOAP_OK;
}

/// Web service operation 'getRunManagerList' (returns error code or SOAP_OK)
int daqppService::getRunManagerList(int dummy, struct ns1__getRunManagerListResponse &_param_2)
{
    ns1__DAQidList *idlist = static_cast<ns1__DAQidList *>(instantiate_type<ns1__DAQidList>(this));
    _param_2.idlist = idlist;
    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    DAQpp::RunManagerMap::const_iterator ip;
    const DAQpp::RunManagerMap &mmap = daqm->RunMmap();
    for (ip = mmap.begin(); ip != mmap.end(); ++ip)
    {
        idlist->name.push_back(ip->first);
    }
    return SOAP_OK;
}

int daqppService::getRunManagerModules(std::string daqid, struct ns1__getRunManagerModulesResponse &_param_7)
{
    ns1__DAQidList *idlist = static_cast<ns1__DAQidList *>(instantiate_type<ns1__DAQidList>(this));
    _param_7.idlist = idlist;
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(daqid);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << daqid << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "getRunManagerModules",  SOAP_USER_ERROR);
    }

    ObserverList::iterator ip;
    DAQpp::RunCommand *rc = (*runm)();
    for (ip = rc->begin(); ip!=rc->end(); ++ip )
    {
        DAQpp::Module *m = (DAQpp::Module *)(*ip);
        idlist->name.push_back( m->get_id() );
    }
    return SOAP_OK;
}

/// Web service operation 'getParNames' (returns error code or SOAP_OK)
int daqppService::getParameterNames(std::string daqid, struct ns1__getParameterNamesResponse &_param_3)
{
    ns1__DAQidList *namelist = static_cast<ns1__DAQidList *>(instantiate_type<ns1__DAQidList>(this));
    _param_3.idlist = namelist;
    DAQpp::DAQObject *obj = find_object(daqid);
    if (!obj)
    {
        std::ostringstream ostr;
        ostr << "DAQObject " << daqid << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "getParameterNames",  SOAP_USER_ERROR);
    }
    DAQpp::ParameterHolder::iterator ip;
    for (ip = obj->begin(); ip != obj->end(); ++ip)
    {
        if ( ip->first != "__PyObject__" )
            namelist->name.push_back(ip->first);
    }
    return SOAP_OK;
}

/// Web service operation 'getStatus' (returns error code or SOAP_OK)
int daqppService::getStatus(std::string daqid, struct ns1__getStatusResponse &_param_4)
{
    ns1__Status *status = static_cast<ns1__Status *>(instantiate_type<ns1__Status>(this));
    _param_4.status = status;
    DAQpp::DAQObject *obj = find_object(daqid);
    if (!obj)
    {
        std::ostringstream ostr;
        ostr << "DAQObject " << daqid << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "getStatus",  SOAP_USER_ERROR);
    }

    const DAQpp::RunState &state = obj->GetStatus();
    status->name = state.get_name();
    status->daqid = daqid;
    status->nexpected = state.n_expected();
    status->ntrigger = state.n_trigger();
    status->nwritten = state.n_written();
    status->eff = state.eff();
    status->rate = state.rate(true);
    status->throughput = state.get_throughput();
    status->status = (ns1__StatusType) state.get_status();
    status->time = state.time();
    return SOAP_OK;

}

/// Web service operation 'getPar' (returns error code or SOAP_OK)
int daqppService::getParameter(ns1__ParameterID *id, struct ns1__getParameterResponse &_param_5)
{
    if (!id)
    {
        std::ostringstream ostr;
        ostr << "No ParameterID provided";
        return soap_set_sender_error(this, ostr.str().c_str(), "getParamter", SOAP_USER_ERROR);
    }
    ns1__Parameter *parvalue = static_cast<ns1__Parameter *>(instantiate_type<ns1__Parameter>(this));
    _param_5.param = parvalue;
    DAQpp::DAQObject *obj = find_object(id->holderID);
    if (!obj)
    {
        std::ostringstream ostr;
        ostr << "DAQObject " << id->holderID << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "getParameter",  SOAP_USER_ERROR);
    }

    DAQpp::Parameter *par = 0;
    parvalue->daqid = id->holderID;
    parvalue->value = soap_new_ns1__ParameterValue(this,  -1);
    parvalue->value->value = soap_new_ns1__valueType(this, -1);
    std::string name, type;
    try
    {
        xsd__anyType *any;
        par = obj->get_par(id->parName.c_str());
        any = get_parameter(this, par);
        type = parse_type_name(par, name);
        parvalue->value->name = id->parName;
        parvalue->value->type = type;
        parvalue->value->value->item.push_back(any);
    }
    catch (DAQpp::ParameterNonExistent &e)
    {
        DAQpp::ostream os;
        os << DAQpp::loglevel(DAQpp::Log::error)
           << "DAQObject " << id->holderID
           << " does not have parameter " << id->parName << std::endl;

        std::ostringstream ostr;
        ostr << "DAQObject " << id->holderID << " does not have parameter " << id->parName;
        return soap_set_sender_error(this, ostr.str().c_str(), "getParameter",  SOAP_USER_ERROR);
    }
    return SOAP_OK;

}

/// Web service operation 'setPar' (returns error code or SOAP_OK)
int daqppService::setParameter(ns1__Parameter *par, int &rc)
{
    if (!par)
    {
        std::ostringstream ostr;
        ostr << "No Parameter provided";
        return soap_set_sender_error(this, ostr.str().c_str(), "setParamter", SOAP_USER_ERROR);
    }

    DAQpp::DAQObject *obj = find_object(par->daqid);
    if (!obj)
    {
        std::ostringstream ostr;
        ostr << "DAQObject " << par->daqid << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "setParameter",  SOAP_USER_ERROR);
    }

    DAQpp::Parameter *dpar = 0;
    try
    {
        if (!par->value)
            return SOAP_TYPE;

        if (!par->value->value)
            return SOAP_TYPE;

        if (par->value->value->item.empty())
            return SOAP_TYPE;

        dpar = obj->get_par(par->value->name.c_str());
        set_parameter(par, dpar);
    }
    catch (DAQpp::ParameterNonExistent &e) {
        DAQpp::ostream os;
        os << DAQpp::loglevel(DAQpp::Log::error)
           << "Object " << par->daqid << " does not have any parameter named " << par->value->name
           << std::endl;

        std::ostringstream ostr;
        ostr << "DAQObject " << par->daqid << " does not have parameter " << par->value->name;
        return soap_set_sender_error(this, ostr.str().c_str(), "setParameter",  SOAP_USER_ERROR);
    }
    return SOAP_OK;

}


/// Web service operation 'activateRunManager' (returns error code or SOAP_OK)
int daqppService::activateRunManager(std::string id, int &rc)
{
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << id << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "activateRunManager",  SOAP_USER_ERROR);
    }

    runm->Activate();
    return SOAP_OK;
}

int daqppService::setMaxEvents(ns1__MaxEvents *mxevts, int &rc)
{
    if (!mxevts)
    {
        std::ostringstream ostr;
        ostr << "NULL MaxEvent pointer";
        return soap_set_sender_error(this, ostr.str().c_str(), "setMaxEvents", SOAP_USER_ERROR);
    }
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(mxevts->daqid);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << mxevts->daqid << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "setMaxEvents",  SOAP_USER_ERROR);
    }
    runm->set_max_events(mxevts->nevts);

    return SOAP_OK;
}

/// Web service operation 'Start' (returns error code or SOAP_OK)
int daqppService::StartRunManager(std::string id, int &rc)
{
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << id << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "StartRunManager",  SOAP_USER_ERROR);
    }

    if (!runm->isActive())
        runm->Activate();

    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    daqm->startRunManager(runm);
    //    runm->Start();
    return SOAP_OK;
}

/// Web service operation 'Stop' (returns error code or SOAP_OK)
int daqppService::StopRunManager(std::string id, int &rc)
{
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << id << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "StopRunManager",  SOAP_USER_ERROR);
    }

    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    daqm->stopRunManager(runm);
    //    runm->Stop();
    return SOAP_OK;
}

/// Web service operation 'GetReady' (returns error code or SOAP_OK)
int daqppService::GetReady(std::string id, int &rc)
{
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << id << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "GetReady",  SOAP_USER_ERROR);
    }

    runm->GetReady();
    return SOAP_OK;
}

/// Web service operation 'Pause' (returns error code or SOAP_OK)
int daqppService::Pause(std::string id, int &rc)
{
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << id << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "Pause",  SOAP_USER_ERROR);
    }

    runm->Pause();
    return SOAP_OK;
}

/// Web service operation 'Continue' (returns error code or SOAP_OK)
int daqppService::Continue(std::string id, int &rc)
{
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << id << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "Continue",  SOAP_USER_ERROR);
    }

    runm->Continue();
    return SOAP_OK;
}

/// Web service operation 'Reset' (returns error code or SOAP_OK)
int daqppService::Reset(std::string id, int &rc)
{
    DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!runm)
    {
        std::ostringstream ostr;
        ostr << "RunManager " << id << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "Reset",  SOAP_USER_ERROR);
    }

    runm->Reset();
    return SOAP_OK;
}

int daqppService::getMonitorData(std::string daqid, xsd__base64Binary &monitor_data)
{
    DAQpp::DAQObject *obj = find_object(daqid);
    if (!obj)
    {
        std::ostringstream ostr;
        ostr << "DAQObject " << daqid << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "getMonitorData",  SOAP_USER_ERROR);
    }

    char *data;
    int size;

    if (obj->get_monitor_data(size, &data))
    {
        monitor_data.__size = size;
        monitor_data.__ptr = (unsigned char *)soap_malloc(this, size);
        memcpy(monitor_data.__ptr, data, monitor_data.__size);
        delete [] data;
    }

    return SOAP_OK;
}

int daqppService::resetMonitorData(std::string daqid, int &resetMonitorDataResponse)
{
    DAQpp::DAQObject *obj = find_object(daqid);
    if (!obj)
    {
        std::ostringstream ostr;
        ostr << "DAQObject " << daqid << " does not exist";
        return soap_set_sender_error(this, ostr.str().c_str(), "resetMonitorData",  SOAP_USER_ERROR);
    }

    obj->reset_monitor_data();
    return SOAP_OK;

}

int daqppService::sendUpdate(int sendUpdateRequest, struct ns1__sendUpdateResponse &request)
{

    /** TODO:
     *  Have a MsgManager that intercepts changes in parameters and Status (the latter mainly for
     *  RunManagers). We keep a list of changes:
     *    1. daqid, parid for parameters
     *    2. daqid for run managers
     *
     *  When the user call this function we build up the Update message based on the current
     *  values of the DAQObject Status of Parameters in the list.
     *
     *  We do not want to repeat "items" in the list, so a given parameter in a given object will
     *  only appear once.
     */

    request.update = instantiate_type<ns1__Update>(this);
    request.update->parameters = instantiate_type<ns1__ParameterUpdate>(this);
    request.update->status = instantiate_type<ns1__StatusUpdate>(this);

    DAQpp::MsgManager::Notification::const_iterator ip;

    // Parameters
    mngr->acquire();

    for (ip=mngr->begin_par(); ip!=mngr->end_par(); ++ip)
    {
        ns1__ParameterID id;
        ns1__getParameterResponse out;
        std::string iid = ip->first;
        int pos = iid.find('\t');
        id.holderID  = iid.substr(0, pos);
        id.parName = ip->second;

        getParameter(&id, out);
        request.update->parameters->parameter.push_back( out.param );
    }
    mngr->release();

    // Status
    mngr->acquire();
    for (ip=mngr->begin_stat(); ip!=mngr->end_stat(); ++ip)
    {
        ns1__getStatusResponse out;
        getStatus(ip->first, out);
        request.update->status->status.push_back( out.status );
    }
    mngr->release();


    // clear processed notifications
    mngr->acquire();
    mngr->clear_notifications();
    mngr->release();
    return SOAP_OK;
}

/// Web service operation 'stopServer' (returns error code or SOAP_OK)
int daqppService::stopServer(int stopServerRequest, int &stopServerResponse)
{
    exit(0);
}
