//#define DAQpp_DEBUG 1
#include <Python.h>
#include <pythread.h>
#include "DAQobserver.h"
#include <iostream>
#include <DAQ++/DAQmanager.h>
#include "pydaqpp_debug.h"
#include "pydaqpp_types.h"

/**
 * Receives the messages from the DAQmanager.
 */
void DAQobserver::update(Observable *o, Argument *arg)
{
    DAQpp::Module *md;
    DAQpp::RunManager *rm;
    //DAQpp::DAQmanager *daqm;
    Arg< DAQpp::DAQmanager::newObjArg> &msg =*static_cast< Arg< DAQpp::DAQmanager::newObjArg> *>(arg);

    WHOAMI;

    switch (msg().first)
    {
        case DAQpp::DAQmanager::startDAQ:
            info_msg("...startDAQ\n");
            break;
        case DAQpp::DAQmanager::stopDAQ:
            info_msg("...stopDAQ\n");
            break;

        case DAQpp::DAQmanager::rmModule:
            md = dynamic_cast<DAQpp::Module*>((DAQpp::DAQObject *)msg().second);
            if (md)
            {
                info_msg("...rmModule\n");
                info_msg("...deleting module %s\n", (const char *)md->get_id());
                std::pair<ObjList::iterator,ObjList::iterator> range = modules.equal_range(md);
                ObjList::iterator ip;
                for (ip=range.first; ip!=range.second; ++ip)
                    MODULE(ip->second)=0;
            }
            break;
        case DAQpp::DAQmanager::rmRunManager:
            info_msg("...rmRunManager\n");
            rm = dynamic_cast<DAQpp::RunManager*>((DAQpp::DAQObject *)msg().second);
            if (rm)
            {
                info_msg("...deleting RunManager %s\n", (const char *)rm->get_id());
                std::pair<ObjList::iterator,ObjList::iterator> range = run_managers.equal_range(rm);
                ObjList::iterator ip;
                for (ip=range.first; ip!=range.second; ++ip)
                {
                    RUN_MANAGER(ip->second)=0;
                }
            }
            break;
        default:
            info_msg("...Unknown message: %x\n", msg().first);
            break;
    }
}
DAQobserver::DAQobserver()
{
    pthread_mutex_init(&_mtx, 0);
}
DAQobserver::~DAQobserver()
{
    WHOAMI;
    pthread_mutex_destroy(&_mtx);
}

PyGILState_STATE DAQobserver::start()
{
    WHOAMI;
//    PyThreadState *tstate = PyThreadState_GET();
//    info_msg("tstate is %x\n",tstate);
//    if (tstate==0)
//    {
        PyGILState_STATE gs = PyGILState_Ensure();
//        pthread_mutex_lock(&_mtx);
//        gstate.push( gs );
//        pthread_mutex_unlock(&_mtx);
        return gs;
//    }
}

void DAQobserver::stop(PyGILState_STATE gs)
{
    WHOAMI;
    bool empty;
//    pthread_mutex_lock(&_mtx);
//    empty = gstate.empty();
//    pthread_mutex_unlock(&_mtx);
//    if ( !empty )
//    {
//        pthread_mutex_lock(&_mtx);
//        gs = gstate.top();
//        gstate.pop();
//        pthread_mutex_unlock(&_mtx);
        PyGILState_Release(gs);
//    }
}

DAQobserver *DAQobserver::_theObserver=0;

void DAQobserver::remove_module_ref(PyObject *obj)
{
    ObjList::iterator ip;
    for (ip=modules.begin(); ip!=modules.end(); ++ip)
    {
        if (ip->second == obj )
        {
            info_msg("Removing reference for module %s\n", (const char *)MODULE(ip->second)->get_id());
            modules.erase(ip);
        }
    }
}

#include "ModuleDecRef.h"

void DAQobserver::remove_runmanager_ref(PyObject *obj)
{
    ObjList::iterator ip;
    for (ip=run_managers.begin(); ip!=run_managers.end(); ++ip)
    {
        if (ip->second == obj )
        {
            info_msg("Removing referenc for run manager %s\n",(const char *) RUN_MANAGER(ip->second)->get_id());
            run_managers.erase(ip);
        }
    }
    // Now check if there is any other reference to this RUnManager
    // somewhere in Python
    ip = run_managers.find(RUN_MANAGER(obj));
    if (ip==run_managers.end())
    {
        // No more RunManagers, so we should free the modules
        DAQpp::RunManager *rm= RUN_MANAGER(obj);
        if (rm)
        {
            info_msg("No more instances of this RunManager [%s] exist...\n",(const char *) rm->get_id());
            rm->for_each_module(free_modules(rm) );
        }
    }
}
void daqpp_observer_start()
{
//    DAQobserver::theObserver()->start();
}
void daqpp_observer_stop()
{
//    DAQobserver::theObserver()->stop();
}

