/* -*- mode: c++ -*- */
#ifndef __DAQobserver_h__
#define __DAQobserver_h__

#include <DAQ++/ObserverModel.h>
#include <map>
#include <Python.h>

/**
 * This class tries to control the threads in python. 
 * It is built to be a singleton, so that there is only one
 * instance of the object.
 * 
 * It derives from Observer and registers to the DAQmanager 
 * startDAQ and stopDAQ signals. At startDAQ it will create a
 * new ThreadHandler to lock the python state. When stopDAQ arrives
 * it clears the ThreadHandler.
 * 
 * To lock the python GIL one uses the start and stop methods that
 * delegate on the ThreadHandler
 */
#include <stack>
#include <pthread.h>

#if (PY_VERSION_HEX < 0x02030000)
typedef PyThreadState *PyGILState_STATE;
#define PyGILState_Ensure PyEval_SaveThread
#define PyGILState_Release PyEval_RestoreThread
typedef int Py_ssize_t;
#endif

class DAQobserver : public Observer
{
    private:
        static DAQobserver *_theObserver;
        std::stack<PyGILState_STATE> gstate;
        pthread_mutex_t _mtx;
    protected:
        DAQobserver();

        DAQobserver(const DAQobserver &)
        {
        }

        typedef std::multimap<void *, PyObject *> ObjList;
        ObjList modules;
        ObjList run_managers;

    public:
        static DAQobserver *theObserver()
        {
            static DAQobserver it;
            return &it;
        }
        virtual ~DAQobserver();
        void update(Observable *o, Argument *arg);

        PyGILState_STATE start();
        void stop(PyGILState_STATE);

        void add_module(void *ptr, PyObject *obj)
        {
            modules.insert(ObjList::value_type(ptr, obj) );
        }
        void remove_module_ref(PyObject *obj);

        void add_runmanager(void *ptr, PyObject *obj)
        {
            run_managers.insert(ObjList::value_type(ptr, obj) );
        }
        void remove_runmanager_ref(PyObject *obj);
};

#endif
