//#define DAQpp_DEBUG 1
#include <Python.h>
#include <iostream>
#include <iomanip>
#include <sstream>
#include "pydaqpp.h"
#include "pydaqpp_debug.h"
#include <DAQ++/GTimer.h>

using namespace DAQpp;

typedef struct {
      PyObject_HEAD
      /* Type-specific fields go here. */
      PyObject *dict;
      GTimer *timer;
} daqpp_GTimer;

static PyObject *
gtimer_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
   daqpp_GTimer *self;
   WHOAMI;

   self = PyObject_New(daqpp_GTimer,type);
   if (self!=NULL) {
      self->timer  = 0;
      self->dict = PyDict_New();
   }
   return (PyObject *)self;
}


static void
gtimer_dealloc(daqpp_GTimer* self)
{
   WHOAMI;
   delete self->timer;
   self->timer = 0;
   Py_DECREF(self->dict);
   self->ob_type->tp_free((PyObject*)self);
}

static int
gtimer_init(daqpp_GTimer *self, PyObject *args, PyObject *kwargs) {
   WHOAMI;
   self->timer = new  GTimer();
   return 0;
}

static PyObject *
gtimer_reset(daqpp_GTimer *self, PyObject *args, PyObject *kwargs) {
   WHOAMI;
   self->timer->reset();
   Py_INCREF(Py_None);
   return Py_None;
}

static PyObject *
gtimer_start(daqpp_GTimer *self, PyObject *args, PyObject *kwargs) {
   WHOAMI;
   self->timer->start();
   Py_INCREF(Py_None);
   return Py_None;
}

static PyObject *
gtimer_stop(daqpp_GTimer *self, PyObject *args, PyObject *kwargs) {
   WHOAMI;
   self->timer->stop();
   Py_INCREF(Py_None);
   return Py_None;
}

static PyObject *
gtimer_is_active(daqpp_GTimer *self, PyObject *args, PyObject *kwargs) {
   WHOAMI;
   return PyInt_FromLong((long)self->timer->is_active());
}

static PyObject *
gtimer_call(daqpp_GTimer *self, PyObject *args, PyObject *kwargs) {
   WHOAMI;
   return PyFloat_FromDouble( (*self->timer)() );
}

static PyObject *
gtimer_wait(daqpp_GTimer *self,PyObject *args, PyObject *kwargs) {
   WHOAMI;
   double secs;
   if (!PyArg_ParseTuple(args,"d:GTimer.wait",&secs))
      return NULL;

   struct timeval t;
   double frac;
   frac = fmod(secs, 1.0);
   secs = floor(secs);
   t.tv_sec = (long)secs;
   t.tv_usec = (long)(frac*1000000.0);
   Py_BEGIN_ALLOW_THREADS
   if (select(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, &t) != 0) {
       Py_BLOCK_THREADS
       PyErr_SetFromErrno(PyExc_IOError);
       return NULL;
   }   
   Py_END_ALLOW_THREADS
   
   //usleep(usec);

   Py_INCREF(Py_None);
   return Py_None; 
}

static PyMethodDef gtimer_methods[] = {
   {"reset",(PyCFunction)gtimer_reset, METH_NOARGS,"Resets the timer."},
   {"start",(PyCFunction)gtimer_start,METH_NOARGS,"Starts the timer"}, 
   {"stop",(PyCFunction)gtimer_stop,METH_NOARGS,"Stops the timer"},
   {"is_active",(PyCFunction)gtimer_is_active,METH_NOARGS,"Tells if the timer is active"}, 
   {"wait",(PyCFunction)gtimer_wait,METH_VARARGS,"Sleeps for the given usec"},
   {NULL}  /* Sentinel */
};
static PyMemberDef gtimer_members[] = {
    {NULL}  /* Sentinel */
};

static PyTypeObject GTimer_Type = {
   PyObject_HEAD_INIT(NULL)
   0,                         /*ob_size*/
   "daqpp.GTimer",             /*tp_name*/
   sizeof(daqpp_GTimer),       /*tp_basicsize*/
   0,                         /*tp_itemsize*/
   (destructor)gtimer_dealloc,                         /*tp_dealloc*/
   0,                         /*tp_print*/
   0,                         /*tp_getattr*/
   0,                         /*tp_setattr*/
   0,                         /*tp_compare*/
   (reprfunc)0,                         /*tp_repr*/
   0,                         /*tp_as_number*/
   0,                         /*tp_as_sequence*/
   0,                         /*tp_as_mapping*/
   0,                         /*tp_hash */
   (ternaryfunc)gtimer_call,                         /*tp_call*/
   (reprfunc)0,                 /*tp_str*/
   PyObject_GenericGetAttr,                         /*tp_getattro*/
   PyObject_GenericSetAttr,                         /*tp_setattro*/
   0,                         /*tp_as_buffer*/
   Py_TPFLAGS_DEFAULT| Py_TPFLAGS_BASETYPE,
   "GTimer wrapper",           /* tp_doc */
   0,		               /* tp_traverse */
    0,		               /* tp_clear */
    0,		               /* tp_richcompare */
    0,		               /* tp_weaklistoffset */
    0,		               /* tp_iter */
    0,		               /* tp_iternext */
    gtimer_methods,             /* tp_methods */
    gtimer_members,             /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    offsetof(daqpp_GTimer, dict),                         /* tp_dictoffset */
    (initproc)gtimer_init,      /* tp_init */
    0,                         /* tp_alloc */
    gtimer_new,                 /* tp_new */
};

void register_gtimer(PyObject *dict) {
   daqpp_register_class(dict,"GTimer",
			&GTimer_Type,
			0);
}
