//#define DAQpp_DEBUG 1
#include "pydaqpp.h"
#include <iostream>
#include <sstream>
#include <string>
#include <cstring>
#include <DAQ++/Module.h>
#include "DAQobserver.h"
#include "Observer.h"
#include "pydaqpp_importSequence.h"
#include "pydaqpp_types.h"
#include "pydaqpp_objects.h"
#include "pydaqpp_buffer.h"
#include "pydaqpp_debug.h"


#define CHECK_PTR(self) {                                           \
   if ( MODULE(self)==0 ) {                                         \
      std::ostringstream ostr;                                      \
      ostr << __FUNCTION__ << ": DAQpp::Module pointer is NULL";    \
      PyErr_SetString(DAQpp_Error,ostr.str().c_str());              \
      return NULL;                                                  \
   }                                                                \
}


// Prepare for a new run
int pyModule::PrepareForRun()
{
    int rc = call_int_method("PrepareForRun", NULL);
    return rc;
}
// Start the run
int pyModule::Start(int mode)
{
    int rc = call_int_method("Start", "i", mode, NULL);
    return rc;
}
// Stop the run
int pyModule::Stop(int mode)
{
    int rc = call_int_method("Stop", "i", mode, NULL);
    return rc;
}
// Pause the run
int pyModule::Pause()
{
    int rc = call_int_method("Pause", NULL);
    return rc;
}
// Resume the run after pause
int pyModule::Continue()
{
    int rc = call_int_method("Continue", NULL);
    return rc;
}
// Send a reset to the module
int pyModule::Reset()
{
    int rc = call_int_method("Reset", NULL);
    return rc;
}
// Initialize the Module
int pyModule::GetReady()
{
    int rc = call_int_method("GetReady", NULL);
    return rc;
}
// Issues a trigger
int pyModule::Trigger()
{
    if ( !GetStatus().isRunning() )
        return 1;
    int rc = call_int_method("Trigger", NULL);
    return rc;
}
// Flush any remaining data on the Module
void pyModule::flush()
{
    call_void_method("flush", NULL);
    return;
}


bool pyModule::hasData()
{
    WHOAMI;
    int rc=call_int_method("hasData", NULL);
    return (bool)rc;
}

int pyModule::check_attributes()
{
    static const char *msg = "Module should define its own hasData and getData methods";
    WHOAMI;
    if (!has_object())
        return 1;

    if (!PyObject_HasAttrString(self(), "hasData") )
    {
        PyErr_SetString(DAQpp_Error, msg);
        return -1;
    }
    if (!PyObject_HasAttrString(self(), "getData") )
    {
        PyErr_SetString(DAQpp_Error, msg);
        return -1;
    }
    return 0;
}

char *pyModule::getData(int &size)
{
    PyObject *obj;
    Py_ssize_t ssize;
    WHOAMI;
    char *ptr= NULL;
    char *cbuff=0;
    size=0;
    if (python_call() || !has_object() )
        return ptr;

    //unsigned int gs = DAQobserver::theObserver()->start();
    PyGILState_STATE gs = PyGILState_Ensure();
    /**
     * We call here the getData method.
     */
    obj = PyObject_CallMethod(self(), "getData", NULL);
    if (obj==0)
    {
        if (PyErr_Occurred())
        {
            PyErr_Print();
            daqpp_print_error();
        }
        goto out;
    }
    if (obj==Py_None)
        goto out;

    // we should get a buffer here or a Py_None
    if ( !PyObject_AsReadBuffer(obj, (const void **)&cbuff, &ssize) )
    {
        size = ssize;
        ptr = allocate(size);
        memcpy(ptr, cbuff, size);
    }
    else if (PySequence_Check(obj) )
    {
        char *str = 0;
        PyErr_Clear(); // this is set by PyObject_AsReadBuffer

        try
        {
            str = import_sequence(obj,ssize);
            size = ssize;
        }
        catch ( SeqException e )
        {
            std::ostringstream ostr;
            ostr << "Module.getData: " << e.what();
            std::cout << "\n***** " << ostr << std::endl;
            PyErr_SetString(DAQpp_Error,ostr.str().c_str());
            goto out;
        }
        ptr = allocate(size);
        memcpy(ptr, str, size);
        PyMem_Free(str);
    }
    else
    {
        PyErr_SetString(DAQpp_Error, "Module.getData: get data should provide a buffer or a sequence");
        size = 0;
        ptr = 0;
    }

    out:
    Py_XDECREF(obj);
    //DAQobserver::theObserver()->stop(gs);
    PyGILState_Release(gs);
    return ptr;
}

bool pyModule::get_monitor_data(int &sz, char **dt)
{
    WHOAMI;
    sz = 0;
    *dt = 0;
    PyObject *ret = call_obj_method("get_monitor_data", NULL);
    if ( ret == Py_None )
    {
        Py_DECREF(ret);
        return false;
    }

    //unsigned int gs = DAQobserver::theObserver()->start();
    PyGILState_STATE gs = PyGILState_Ensure();
    Py_ssize_t ssize;
    char *cbuff=0;

    // we should get a buffer here or a Py_None
    if ( !PyObject_AsReadBuffer(ret, (const void **)&cbuff, &ssize) )
    {
        sz = ssize;
        *dt = new char[ssize];
        memcpy(*dt, cbuff, sz);
    }
    else if (PySequence_Check(ret) )
    {
        char *str = 0;
        PyErr_Clear(); // this is set by PyObject_AsReadBuffer

        try
        {
            str = import_sequence(ret,ssize);
            sz = ssize;
        }
        catch ( SeqException e )
        {
            std::ostringstream ostr;
            ostr << "Module.get_monitor_data: " << e.what();
            std::cout << "\n***** " << ostr << std::endl;
            PyErr_SetString(DAQpp_Error,ostr.str().c_str());
            goto out;
        }
        *dt = new char[sz];
        memcpy(*dt, str, sz);
        PyMem_Free(str);
    }
    else
    {
        PyErr_SetString(DAQpp_Error, "Module.get_monitor_data should provide a buffer or a sequence");
        sz = 0;
        *dt = 0;
    }

out:
    info_msg("...quitting\n");
    Py_DECREF(ret);
    //DAQobserver::theObserver()->stop(gs);
    PyGILState_Release(gs);
    return sz>0;
}
void pyModule::reset_monitor_data()
{
    WHOAMI;
    call_void_method("reset_monitor_data", NULL);
    info_msg("...quitting\n");
    return;
}



static PyObject *module_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    daqpp_Module *self;

    WHOAMI;
    self = (daqpp_Module *)type->tp_alloc(type, 0);
    if (!self)
        return (PyObject *)0;

    self->m = 0;
    return (PyObject *)self;
}


void module_decref(DAQpp::Module *m)
{
    PyDAQpp *pm = dynamic_cast<PyDAQpp *>(m);
    PyObject *obj=0;
    if (pm)
    {
        obj = pm->self();
    }
    else
    {
        try
        {
            obj = DAQpp::get_par_value<PyObject *>( m->get_par("__PyObject__") );
        }
        catch ( DAQpp::ParameterNonExistent ex)
        {
            std::cout << "module_decref: " << ex.what() << std::endl;
        }
    }
    if (obj)
    {
        Py_DECREF( obj );
    }

}

static PyObject *module_refcount(PyObject *obj)
{
    return PyInt_FromLong(obj->ob_refcnt);
}

void terminate_daqpp_module(daqpp_Module *self)
{
    if (MODULE(self))
        DAQobserver::theObserver()->remove_module_ref((PyObject *)self);

}
static void module_dealloc(daqpp_Module* self)
{
    WHOAMI;

    terminate_daqpp_module(self);

    Py_DECREF(self->pH.dict);
    self->pH.ob_type->tp_free((PyObject*)self);
}

static int set_object_pointer(daqpp_Module *self, DAQpp::Module *m)
{
    WHOAMI;
    int rc=0;
    self->pH.h = dynamic_cast<DAQpp::ParameterHolder *>(m);
    self->m = m;

    PyDAQpp *pm = dynamic_cast<PyDAQpp *>(m);
    if (pm)
    {
        pm->set_object((PyObject *)self);
        if (pm->check_attributes()<0)
            rc=-1;
    }

    return rc;
}

int setup_daqpp_module(DAQpp::Module *m, daqpp_Module *self)
{
    int rc=0;
    DAQobserver::theObserver()->add_module((void *)m, (PyObject *)self);
    if (set_object_pointer(self, m))
        rc=-1;

    return rc;
}

static int module_init(daqpp_Module *self, PyObject *args, PyObject *kwargs)
{
    PyObject *in=Py_None;
    PyObject *str;
    int rc = 0;

    WHOAMI;

    if (!PyArg_ParseTuple(args, "O:daqpp.Module.__init__", &in))
        return -1;

    if (in!=Py_None)
    {
        PyObject *str = PyObject_Str(in);
        std::string v = PyString_AsString(str);
        pyModule *m = new pyModule(v);
        rc = setup_daqpp_module(m, self);

        Py_DECREF(str);
    }
    return rc;
}

PyDoc_STRVAR(module_delete_doc,"\
Deletes the DAQpp::Module. This immediatly invalidates the Python\n\
reference and the next access to it will raise a daqpp.DAQError\n\
");
static PyObject *module_delete(daqpp_Module *self)
{
    CHECK_PTR(self);
    if ( MODULE(self))
        delete MODULE(self);

    MODULE(self)=0;
    Py_INCREF(Py_None);
    return Py_None;
}

PyDoc_STRVAR(module_get_id_doc,"Returns the Module id");
static PyObject *module_get_id(daqpp_Module *self)
{
    WHOAMI;
    CHECK_PTR(self);
    DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
    DAQpp::DAQid id = md->get_id();
    PyObject* daqid = PyDAQpp_DAQid_New((const char *)id);
    return daqid;
}

PyDoc_STRVAR(prepareforrun_doc,
"Called at the beginning of each run. The module can make\n\
here any run dependent initialization\n\
");
static PyObject *module_PrepareForRun(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );

    WHOAMI;

    if (pm->external_call())
    {
        rc = PyInt_FromLong(0L);
    }
    else
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->PrepareForRun());
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(modulestart_doc,"Starts the module acquisition\n\
");
static PyObject *module_Start(daqpp_Module *self, PyObject *args, PyObject *kwargs)
{
    CHECK_PTR(self);
    PyObject *rc;
    int mode;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );

    WHOAMI;

    if (pm->external_call())
    {
        rc = PyInt_FromLong(0L);
    }
    else
    {
        if (!PyArg_ParseTuple(args, "i:Module.Start", &mode))
            return NULL;

        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->Start(mode));
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(modulestop_doc,"Stops the module acquisition\n\
");
static PyObject *module_Stop(daqpp_Module *self, PyObject *args, PyObject *kwargs)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    int mode;
    WHOAMI;

    if (pm->external_call())
    {
        rc = PyInt_FromLong(0L);
    }
    else
    {
        if (!PyArg_ParseTuple(args, "i:Module.Stop", &mode))
            return NULL;

        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->Stop(mode));
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(modulepause_doc,"Pauses the module acquisition\n\
");
static PyObject *module_Pause(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    WHOAMI;

    if (pm->external_call())
    {
        rc = PyInt_FromLong(0L);
    }
    else
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->Pause());
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(modulecontinue_doc,"Resumes the module acquisition\n\
");
static PyObject *module_Continue(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    WHOAMI;

    if (pm->external_call())
    {
        rc = PyInt_FromLong(0L);
    }
    else
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->Continue());
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(modulereset_doc,"Resets the module\n\
");
static PyObject *module_Reset(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    WHOAMI;

    if (pm->external_call())
    {
        rc = PyInt_FromLong(0L);
    }
    else
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->Reset());
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(modulegetready_doc,"Configures the module\n\
");
static PyObject *
module_GetReady(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    WHOAMI;

    if (pm->external_call())
    {
        rc = PyInt_FromLong(0L);
    }
    else
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->GetReady() );
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(moduletrigger_doc,
"This is called write before hasData on each event.\n\
It indicates that the DAQ received a trigger.\n\
");
static PyObject *module_Trigger(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    WHOAMI;

    if (pm->external_call())
    {
        rc = PyInt_FromLong(0L);
    }
    else
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->Trigger());
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(moduleflush_doc,"Flushes any data in the module\n\
");
static PyObject *module_flush(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    WHOAMI;

    if (!pm->external_call())
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        md->flush();
        pm->clear_python_call();
    }
    Py_INCREF(Py_None);
    return Py_None;

}


PyDoc_STRVAR(module_is_global_doc,"Says whether the module is in global\n\
");
static PyObject *module_is_global(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    WHOAMI;

    if (pm->external_call())
    {
        rc = Py_None;
        Py_INCREF(rc);
    }
    else
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->is_global());
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(module_is_local_doc,"Says whether the module is in local\n\
");
static PyObject *module_is_local(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject *rc;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    WHOAMI;

    if (pm->external_call())
    {
        rc = Py_None;
        Py_INCREF(rc);
    }
    else
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        pm->set_python_call();
        rc = PyInt_FromLong( md->is_local());
        pm->clear_python_call();
    }
    return rc;
}

PyDoc_STRVAR(module_in_local_doc,"Sets the module in local mode.\n\
");
static PyObject *module_in_local(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );

    WHOAMI;
    if (!pm->external_call())
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        md->in_local();
    }

    Py_INCREF(Py_None);
    return Py_None;
}

PyDoc_STRVAR(module_in_global_doc,"Sets the module in global mode.\n\
");
static PyObject *module_in_global(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );

    WHOAMI;

    if (!pm->external_call())
    {
        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(MODULE(self));
        md->in_global();
    }

    Py_INCREF(Py_None);
    return Py_None;
}


PyDoc_STRVAR(module_getstatus_doc,"Returns the current status of the module.\n\
");
static PyObject *module_GetStatus(daqpp_Module *self)
{
    CHECK_PTR(self);
    PyObject* status = PyDAQpp_RunState_New();
    WHOAMI;

    *RUN_STATE(status) = MODULE(self)->GetStatus();

    return status;
}

PyDoc_STRVAR(module_addObserver_doc,
"Adds an object that will be notified when any parameter changes\n\
");
static PyObject *module_addObserver(daqpp_Module *self, PyObject *args, PyObject *kwargs)
{
    PyObject *obj;
    PyObject *ret=Py_None;
    CHECK_PTR(self);
    if (!PyArg_ParseTuple(args, "O:Module.addObserver", &obj))
        return NULL;

    if (obj!=NULL)
    {
        pyObserver *obs = new pyObserver(obj);
        MODULE(self)->addObserver( *obs );
    }
    Py_INCREF(ret);
    return ret;
}

PyDoc_STRVAR(module_reset_mon_data_doc,
"Resets the monitor data to default values.\n\
");
static PyObject *
module_reset_mon_data(daqpp_Module *self, PyObject *args, PyObject *kwargs)
{
    WHOAMI;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    if (!pm->external_call())
    {
        pm->set_python_call();
        MODULE(self)->reset_monitor_data();
        pm->clear_python_call();
    }
    Py_INCREF(Py_None);
    return Py_None;
}

PyDoc_STRVAR(module_get_mon_data_doc,
"Gets the monitor data into a buffer. Returns a Buffer if data is\n\
available. None otherwise.\n\
");
static PyObject *
module_get_mon_data(daqpp_Module *self, PyObject *args, PyObject *kwargs)
{
    WHOAMI;
    PyObject *ret;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( MODULE(self) );
    if (!pm->external_call())
    {
        pm->set_python_call();
        int size;
        char *data;
        if ( MODULE(self)->get_monitor_data(size, &data) )
        {
            ret = PyDAQpp_Buffer_New('c', size, 0);
            memcpy(((daqpp_buffer *)ret)->item, data, size);
        }
        else
        {
            ret = Py_None;
            Py_INCREF(ret);
        }
        pm->clear_python_call();
    }
    return ret;
}


static PyMethodDef module_methods[] = {
   {"refcnt",(PyCFunction)module_refcount,METH_NOARGS,"."},
   {"delete",(PyCFunction)module_delete, METH_NOARGS,module_delete_doc},
   {"get_id",(PyCFunction)module_get_id, METH_NOARGS,module_get_id_doc},
   {"GetStatus",(PyCFunction)module_GetStatus, METH_NOARGS,module_getstatus_doc},
   {"PrepareForRun",(PyCFunction)module_PrepareForRun,METH_NOARGS,prepareforrun_doc},
   {"Start",(PyCFunction)module_Start,METH_VARARGS,modulestart_doc},
   {"Stop",(PyCFunction)module_Stop,METH_VARARGS,modulestop_doc},
   {"Pause",(PyCFunction)module_Pause,METH_NOARGS,modulepause_doc},
   {"Continue",(PyCFunction)module_Continue,METH_NOARGS,modulecontinue_doc},
   {"Reset",(PyCFunction)module_Reset,METH_NOARGS,modulereset_doc},
   {"GetReady",(PyCFunction)module_GetReady,METH_NOARGS,modulegetready_doc},
   {"Trigger",(PyCFunction)module_Trigger,METH_NOARGS,moduletrigger_doc},
   {"flush",(PyCFunction)module_flush,METH_NOARGS,moduleflush_doc},
   {"is_global",(PyCFunction)module_is_global,METH_NOARGS,module_is_global_doc},
   {"is_local",(PyCFunction)module_is_local,METH_NOARGS,module_is_local_doc},
   {"in_global",(PyCFunction)module_in_global,METH_NOARGS,module_in_global_doc},
   {"in_local",(PyCFunction)module_in_local,METH_NOARGS,module_in_local_doc},
   {"addObserver",(PyCFunction)module_addObserver,METH_VARARGS,module_addObserver_doc},
   {"reset_monitor_data",(PyCFunction)module_reset_mon_data,METH_NOARGS,module_reset_mon_data_doc},
   {"get_monitor_data",(PyCFunction)module_get_mon_data,METH_NOARGS,module_get_mon_data_doc},
   {NULL}  /* Sentinel */
};


PyDoc_STRVAR(moduletype_doc,
"This object represent anything that can provide data. It is usually\n\
controlled by a RunManager that can send him any of the following\n\
commands:\n\
      - int    PrepareForRun(self),\n\
      - int    Start(self,mode),\n\
      - int    Stop(self,mode),\n\
      - int    Pause(self),\n\
      - int    Continue(self),\n\
      - int    Reset(self),\n\
      - int    GetReady(self),\n\
      - bool   hasData(self),\n\
      - buffer getData(self),\n\
      - int    Trigger(self),\n\
      - void   flush(self)\n\
\n\
A module can also mantain a list of named parameters. Parameters \n\
can be added with add_par() and retrieved with get_par(). To change\n\
the value of a given parameter one uses set_par().\n\
\n\
A Module object derived from daqpp.Module needs to implement at least\n\
      - hasData(self): returns True if there is data availble and\n\
                       False otherwise\n\
      - getData(self): returns a buffer like object with the current\n\
                       event data.\n\
\n\
");

#define DEFERRED_ADDRESS(ADDR) 0
PyTypeObject Module_Type = {
   PyObject_HEAD_INIT(NULL)
   0,                         /*ob_size*/
   "daqpp.Module",             /*tp_name*/
   sizeof(daqpp_Module),       /*tp_basicsize*/
   0,                         /*tp_itemsize*/
   (destructor)module_dealloc,/*tp_dealloc*/
   0,                         /*tp_print*/
   0,                         /*tp_getattr*/
   0,                         /*tp_setattr*/
   0,                         /*tp_compare*/
   (reprfunc)0,                         /*tp_repr*/
   0,                         /*tp_as_number*/
   0,                         /*tp_as_sequence*/
   0,                         /*tp_as_mapping*/
   0,                         /*tp_hash */
   (ternaryfunc)0,            /*tp_call*/
   (reprfunc)0,               /*tp_str*/
   0,                         /*tp_getattro*/
   0,                         /*tp_setattro*/
   0,                         /*tp_as_buffer*/
   Py_TPFLAGS_DEFAULT| Py_TPFLAGS_BASETYPE,        /*tp_flags*/
   moduletype_doc,             /* tp_doc */
   0,		               /* tp_traverse */
   0,		               /* tp_clear */
   0,		               /* tp_richcompare */
   0,		               /* tp_weaklistoffset */
   0,		               /* tp_iter */
   0,		               /* tp_iternext */
   module_methods,             /* tp_methods */
   0,             /* tp_members */
   0,                         /* tp_getset */
   DEFERRED_ADDRESS(&ParamHolder_Type), /* tp_base */
   0,                         /* tp_dict */
   0,                         /* tp_descr_get */
   0,                         /* tp_descr_set */
   0,                         /* tp_dictoffset */
   (initproc)module_init,      /* tp_init */
   0,                         /* tp_alloc */
   module_new,                 /* tp_new */
};


void register_module(PyObject *dict)
{
    Module_Type.tp_base= &ParamHolder_Type;
    daqpp_register_class(dict, "Module", &Module_Type, 0);
}

int PyDAQpp_Module_Check(PyObject *O)
{
    return PyType_IsSubtype(O->ob_type, &Module_Type);
}

PyObject *PyDAQpp_Module_New(void *ptr)
{
    WHOAMI;
    daqpp_Module *ret;

    DAQpp::Module *m =dynamic_cast<DAQpp::Module*>((DAQpp::Module*)ptr);
    if (!m)
    {
        PyErr_SetString(DAQpp_Error, "Parameter 1 of Module_New is not a Module");
        Py_INCREF(Py_None);
        return Py_None;
    }

    DAQpp::Parameter *par;
    if ( m->has_par("__PyObject__") )
    {
        par = m->get_par("__PyObject__");
        ret = (daqpp_Module *)DAQpp::get_par_value<PyObject *>(par);
        Py_INCREF( (PyObject *)ret);
    }
    else
    {
        ret = PyObject_New(daqpp_Module,&Module_Type);
        if (ret == NULL)
        {
            PyErr_SetString(DAQpp_Error,"daqpp.Module_New: no memory");
            return NULL;
        }
        ret->pH.dict = PyDict_New();
        par = new DAQpp::Par<PyObject *>( (PyObject *)ret );
        m->add_par("__PyObject__",par);
        Py_INCREF( (PyObject *)ret);

        set_object_pointer((daqpp_Module *)ret,m);

        DAQobserver::theObserver()->add_module((void *)m,(PyObject *)ret);

    }
//    try
//    {
//        par = m->get_par("__PyObject__");
//        ret = (daqpp_Module *)DAQpp::get_par_value<PyObject *>(par);
//        Py_INCREF( (PyObject *)ret);
//    }
//    catch ( DAQpp::ParameterNonExistent ex)
//    {
//        ret = PyObject_New(daqpp_Module,&Module_Type);
//        if (ret == NULL)
//        {
//            PyErr_SetString(DAQpp_Error,"daqpp.Module_New: no memory");
//            return NULL;
//        }
//        ret->pH.dict = PyDict_New();
//        par = new DAQpp::Par<PyObject *>( (PyObject *)ret );
//        m->add_par("__PyObject__",par);
//
//        set_object_pointer((daqpp_Module *)ret,m);
//
//        DAQobserver::theObserver()->add_module((void *)m,(PyObject *)ret);
//    }
//
    return (PyObject *)ret;
}
