#include "pydaqpp.h"
#include <vector>
#include <DAQ++/DAQmanager.h>
#include <DAQ++/ObserverModel.h>
#include <DAQ++/exception.h>
#include <DAQ++/RunState.h>
#include "DAQobserver.h"
#include <typeinfo>
#include <cxxabi.h>
#include <sstream>
#include "pydaqpp_debug.h"
#include "Observer.h"

pyObserver::pyObserver(PyObject *o) : pyfunc(o)  {
   WHOAMI;
   if ( check_attributes() )
      return;

   if (pyfunc)
      Py_INCREF(pyfunc);
}

pyObserver::~pyObserver() {
   if (pyfunc)
   {
      Py_DECREF(pyfunc);
   }
}

int pyObserver::check_attributes() {
   int rc = 0;
   WHOAMI;

   if (!pyfunc)
      return 1;

   if (!PyObject_HasAttrString(pyfunc,"update") ) {
      PyErr_SetString(DAQpp_Error,
		      "Observer should define its own update method");
      return -1;
   }
   return rc;
}



#define PAR_VECTOR_GETTER(type,name,convert_func)                                    \
static PyObject *get_par_vector_##name(Argument *arg)                                \
{                                                                                    \
    Arg< std::vector<type>  > *a = dynamic_cast< Arg< std::vector<type> > *>(arg);   \
    const std::vector<type> &v = (*a)();                                             \
    std::vector<type>::const_iterator ip;                                            \
    PyObject *lst = PyList_New(0);                                                   \
    PyObject *item=0;                                                                \
    for (ip=v.begin();ip!=v.end();++ip) {                                            \
        item = convert_func( *ip );                                                  \
        PyList_Append(lst,item);                                                     \
    }                                                                                \
    return lst;                                                                      \
}

PAR_VECTOR_GETTER(double, double, PyFloat_FromDouble);
PAR_VECTOR_GETTER(float, float, PyFloat_FromDouble);
PAR_VECTOR_GETTER(bool,bool, PyInt_FromLong);
PAR_VECTOR_GETTER(short,short, PyInt_FromLong);
PAR_VECTOR_GETTER(unsigned short,ushort,PyInt_FromLong);
PAR_VECTOR_GETTER(int,int,PyInt_FromLong);
PAR_VECTOR_GETTER(unsigned int,uint, PyInt_FromLong);
PAR_VECTOR_GETTER(long,long, PyInt_FromLong);
PAR_VECTOR_GETTER(unsigned long,ulong, PyInt_FromLong);


PyObject *pyObserver::get_value(Argument *arg) {
	PyObject *obj;

	if ( typeid( *arg ) == typeid( Arg<long>  ) ) {
		Arg<long> *a = dynamic_cast<Arg<long> *>(arg);
		obj = PyLong_FromLong( (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<unsigned long>  ) ) {
		Arg<unsigned long> *a = dynamic_cast<Arg<unsigned long> *>(arg);
		obj = PyLong_FromUnsignedLong( (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<int>  ) ) {
		Arg<int> *a = dynamic_cast<Arg<int> *>(arg);
		obj = PyInt_FromLong( (long) (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<unsigned int>  ) ) {
		Arg<unsigned int> *a = dynamic_cast<Arg<unsigned int> *>(arg);
		obj = PyInt_FromLong( (long) (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<short>  ) ) {
		Arg<short> *a = dynamic_cast<Arg<short> *>(arg);
		obj = PyInt_FromLong( (long) (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<unsigned short>  ) ) {
		Arg<unsigned short> *a = dynamic_cast<Arg<unsigned short> *>(arg);
		obj = PyInt_FromLong( (long) (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<DAQpp::RunState::StatusType>  ) ) {
		Arg<DAQpp::RunState::StatusType> *a = dynamic_cast<Arg<DAQpp::RunState::StatusType> *>(arg);
		obj = PyInt_FromLong( (long) (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<float>  ) ) {
		Arg<float> *a = dynamic_cast<Arg<float> *>(arg);
		obj = PyFloat_FromDouble( (double)(*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<double>  ) ) {
		Arg<double> *a = dynamic_cast<Arg<double> *>(arg);
		obj = PyFloat_FromDouble( (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<char *> ) ) {
		Arg<char *> *a = dynamic_cast< Arg<char *> *>(arg);
		obj = PyString_FromString ( (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<const char *>) ) {
		Arg<const char *> *a = dynamic_cast< Arg<const char *> *>(arg);
		obj = PyString_FromString ( (*a)() );
	}
	else if ( typeid( *arg ) == typeid( Arg<std::string>) ) {
		Arg<std::string> *a = dynamic_cast< Arg<std::string> *>(arg);
		obj = PyString_FromString ( (*a)().c_str() );
	}
	else if ( typeid( *arg ) == typeid( Arg<PyObject *>) ) {
		Arg<PyObject *> *a = dynamic_cast< Arg<PyObject *> *>(arg);
		obj = (*a)();
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<double> >))
	{
		return get_par_vector_double(arg);
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<float> >))
	{
		return get_par_vector_float(arg);
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<bool> >))
	{
		return get_par_vector_bool(arg);
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<int> >))
	{
		return get_par_vector_int(arg);
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<unsigned int> >))
	{
		return get_par_vector_uint(arg);
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<short> >))
	{
		return get_par_vector_short(arg);
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<unsigned short> >))
	{
		return get_par_vector_ushort(arg);
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<long> >))
	{
		return get_par_vector_long(arg);
	}
	else if ( typeid(*arg) == typeid( Arg<std::vector<unsigned long> >))
	{
		return get_par_vector_ulong(arg);
	}
	else if ( typeid(*arg) == typeid( Arg< DAQpp::DAQmanager::newObjArg> ))
	{
		Arg< DAQpp::DAQmanager::newObjArg> &msg =*static_cast< Arg< DAQpp::DAQmanager::newObjArg> *>(arg);
		obj = PyTuple_New(2);
		PyTuple_SET_ITEM(obj,0, PyInt_FromLong(msg().first));
		switch ( msg().first )
		{
			case DAQpp::DAQmanager::startDAQ:
				PyTuple_SET_ITEM(obj, 1, PyString_FromString("startDAQ"));
				break;
		case DAQpp::DAQmanager::stopDAQ:
			PyTuple_SET_ITEM(obj, 1, PyString_FromString("stopDAQ"));
			break;
		default:
			DAQpp::DAQObject *dobj = reinterpret_cast<DAQpp::DAQObject *>( msg().second );
			PyTuple_SET_ITEM(obj, 1, PyString_FromString( (const char *)dobj->get_id() ));
			break;
		}
	}
	else {
		//      int status;
		//      const std::type_info  &ti = typeid( *arg );
		PyObject *ret = Py_None;
		//      std::ostringstream ostr;
		//      ostr << "daqpp.Observer: cannot handle "
		//	   << abi::__cxa_demangle(ti.name(), 0, 0, &status)
		//	 ;
		//      PyErr_SetString(DAQpp_Error,ostr.str().c_str());
		Py_INCREF(ret);
		return ret;
	}
	return obj;
}

void pyObserver::update(Observable *o, Argument *arg) {
   PyObject *obj;
   PyObject *val, *tmp=0;
   WHOAMI;
   if (!pyfunc)
      return;

   //unsigned int gs = DAQobserver::theObserver()->start();
   PyGILState_STATE gs= PyGILState_Ensure();
   DAQpp::Parameter *par = dynamic_cast<DAQpp::Parameter *>(o);
   val = get_value(arg);
   if (par)
   {
       PyObject *tuple = PyTuple_New(3);
       Py_IncRef(val); // tuple->SetItem steals a reference...
       PyTuple_SetItem( tuple, 0, val);
       PyTuple_SetItem( tuple, 1, PyString_FromString( par->get_name().c_str() ));
       PyTuple_SetItem( tuple, 2, PyString_FromString( (const char *)par->get_id() ));
       tmp = val;
       val = tuple;
   }
   PyObject *name = PyString_FromString("update");
   obj =  PyObject_CallMethodObjArgs(pyfunc,name,val,NULL);
   if ( obj )
   {
      Py_DECREF(obj);
   }
   Py_DECREF(name);
   Py_XDECREF(val);
   //DAQobserver::theObserver()->stop(gs);
   PyGILState_Release(gs);
   return;
}

static PyObject *
observer_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
   daqpp_Observer *self;

   WHOAMI;
   self = PyObject_New(daqpp_Observer,type);
   if (!self)
      return (PyObject *)0;


   self->obs   = 0;
   self->dict = PyDict_New();
   return (PyObject *)self;
}

static void
observer_dealloc(daqpp_Observer* self)
{
   WHOAMI;
   Py_DECREF(self->dict);
   self->ob_type->tp_free((PyObject*)self);
}

static int
set_object_pointer(daqpp_Observer *self, pyObserver *o) {
   int rc=0;
   WHOAMI;
   self->obs = o;

   return rc;
}

static int
observer_init(daqpp_Observer *self,PyObject *args, PyObject *kwargs) {
   int rc=0;
   PyObject *in=NULL;

   if (!PyArg_ParseTuple(args,"|O:Observer.__init__",&in))
      return -1;

   pyObserver *m = new  pyObserver(in);
   if (set_object_pointer(self,m))
      rc=-1;

   return rc;
}

static PyMethodDef observer_methods[] = {
   {NULL}  /* Sentinel */
};

static PyMemberDef observer_members[] = {
    {NULL}  /* Sentinel */
};

PyDoc_STRVAR(observertype_doc,
"A Observer is an object that is notified whenever something is changed \n\
in an Observable. An example of an Observable is a RunState object. \n\
An Observer has to register to the Observable in order to get notifications.\n\
\n\
Notifications are send by invoquing the update method in the Observer\n\
whose argument is a python object corresponding to the argument sent\n\
by the Observable\n\
"
);

static PyTypeObject Observer_Type = {
   PyObject_HEAD_INIT(NULL)
   0,                         /*ob_size*/
   "daqpp.Observer",             /*tp_name*/
   sizeof(daqpp_Observer),       /*tp_basicsize*/
   0,                         /*tp_itemsize*/
   (destructor)observer_dealloc,                         /*tp_dealloc*/
   0,                         /*tp_print*/
   0,                         /*tp_getattr*/
   0,                         /*tp_setattr*/
   0,                         /*tp_compare*/
   (reprfunc)0,                         /*tp_repr*/
   0,                         /*tp_as_number*/
   0,                         /*tp_as_sequence*/
   0,                         /*tp_as_mapping*/
   0,                         /*tp_hash */
   (ternaryfunc)0,                         /*tp_call*/
   (reprfunc)0,                 /*tp_str*/
   PyObject_GenericGetAttr,                         /*tp_getattro*/
   PyObject_GenericSetAttr,                         /*tp_setattro*/
   0,                         /*tp_as_buffer*/
   Py_TPFLAGS_DEFAULT| Py_TPFLAGS_BASETYPE,        /*tp_flags*/
   observertype_doc,           /* tp_doc */
       0,		               /* tp_traverse */
    0,		               /* tp_clear */
    0,		               /* tp_richcompare */
    0,		               /* tp_weaklistoffset */
    0,		               /* tp_iter */
    0,		               /* tp_iternext */
    observer_methods,             /* tp_methods */
    observer_members,             /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    offsetof(daqpp_Observer, dict),                         /* tp_dictoffset */
    (initproc)observer_init,      /* tp_init */
    0,                         /* tp_alloc */
    observer_new,                 /* tp_new */
};


void register_observer(PyObject *dict) {
   daqpp_register_class(dict,"Observer",
			&Observer_Type,
			0);
}

int PyObserver_Check(PyObject *o) {
   return PyObject_TypeCheck(o, &Observer_Type);
}
