#include "pydaqpp.h"
#include <iostream>
#include <DAQ++/Progress.h>

using namespace DAQpp;

typedef struct {
      PyObject_HEAD
      /* Type-specific fields go here. */
      Progress *state;
}  daqpp_Progress;


static PyObject *
progress_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
   daqpp_Progress *self;
   self = PyObject_New(daqpp_Progress ,type);
   if (self!=NULL) {
      self->state  = 0;
   }
   return (PyObject *)self;
}

static void
progress_dealloc(daqpp_Progress* self)
{
   delete self->state;
   self->ob_type->tp_free((PyObject*)self);
}

static int
progress_init(daqpp_Progress *self, PyObject *args, PyObject *kwargs) {
   double a,b;
   unsigned long opts;
   if (!PyArg_ParseTuple(args,"ddk",&a,&b,&opts))
      return -1;
   self->state = new  Progress(a,b,opts);
   return 0;
}

static PyObject *
progress_start(daqpp_Progress *self) {
   self->state->start();
   Py_INCREF(Py_None);
   return Py_None;
}
static PyObject *
progress_stop(daqpp_Progress *self) {
   self->state->start();
   Py_INCREF(Py_None);
   return Py_None;
}
static PyObject *
progress_timer_start(daqpp_Progress *self) {
   self->state->timer_start();
   Py_INCREF(Py_None);
   return Py_None;
}
static PyObject *
progress_timer_stop(daqpp_Progress *self) {
   self->state->timer_stop();
   Py_INCREF(Py_None);
   return Py_None;
}
static PyObject *
progress_timer_reset(daqpp_Progress *self) {
   self->state->timer_reset();
   Py_INCREF(Py_None);
   return Py_None;
}

static PyObject *
progress_update(daqpp_Progress *self,PyObject *args, PyObject *kwargs) {
   double val;
   if (!PyArg_ParseTuple(args,"d",&val))
      return NULL;
   self->state->update(val);
   Py_INCREF(Py_None);
   return Py_None;
}
static PyObject *
progress_reset(daqpp_Progress *self,PyObject *args, PyObject *kwargs) {
   double a,b;
   if (!PyArg_ParseTuple(args,"dd",&a,&b))
      return NULL;
   self->state->reset(a,b);
   Py_INCREF(Py_None);
   return Py_None;
}

static PyObject *
progress_set_update(daqpp_Progress *self,PyObject *args, PyObject *kwargs) {
   unsigned long usec;
   if (!PyArg_ParseTuple(args,"k",&usec))
      return NULL;
   self->state->set_update(usec);
   Py_INCREF(Py_None);
   return Py_None;
}
static PyObject *
progress_set_options(daqpp_Progress *self,PyObject *args, PyObject *kwargs) {
   unsigned long opts;
   if (!PyArg_ParseTuple(args,"k",&opts))
      return NULL;
   self->state->set_options(opts);
   Py_INCREF(Py_None);
   return Py_None;
}
static PyObject *
progress_Vmin(daqpp_Progress *self) {
   return PyFloat_FromDouble( self->state->Vmin() );
}
static PyObject *
progress_Vmax(daqpp_Progress *self) {
   return PyFloat_FromDouble( self->state->Vmax() );
}
static PyObject *
progress_Value(daqpp_Progress *self) {
   return PyFloat_FromDouble( self->state->Value() );
}
static PyObject *
progress_Percent(daqpp_Progress *self) {
   return PyFloat_FromDouble( self->state->Percent() );
}
static PyObject *
progress_time(daqpp_Progress *self) {
   return PyFloat_FromDouble( self->state->time() );
}
static PyObject *
progress_mem(daqpp_Progress *self) {
   return PyFloat_FromDouble( self->state->mem() );
}

static PyObject *
progress_str(daqpp_Progress *self) {
   return PyString_FromString( self->state->str().c_str() );
}

static PyMethodDef progress_methods[] = {
   {"start",(PyCFunction)progress_start,METH_NOARGS,0},
   {"stop",(PyCFunction)progress_stop,METH_NOARGS,0},
   {"timer_start",(PyCFunction)progress_timer_start,METH_NOARGS,0},
   {"timer_stop",(PyCFunction)progress_timer_stop,METH_NOARGS,0},
   {"timer_reset",(PyCFunction)progress_timer_reset,METH_NOARGS,0},
   {"update",(PyCFunction)progress_update,METH_VARARGS,0},
   {"reset",(PyCFunction)progress_reset,METH_VARARGS,0},
   {"set_update",(PyCFunction)progress_set_update,METH_VARARGS,0},
   {"set_options",(PyCFunction)progress_set_options,METH_VARARGS,0},
   {"Vmin",(PyCFunction)progress_Vmin,METH_NOARGS,0},
   {"Vmax",(PyCFunction)progress_Vmax,METH_NOARGS,0},
   {"Value",(PyCFunction)progress_Value,METH_NOARGS,0},
   {"Percent",(PyCFunction)progress_Percent,METH_NOARGS,0},
   {"time",(PyCFunction)progress_time,METH_NOARGS,0},
   {"mem",(PyCFunction)progress_mem,METH_NOARGS,0},
   {"str",(PyCFunction)progress_str,METH_NOARGS,0},
   {NULL}  /* Sentinel */
};
static PyMemberDef progress_members[] = {
   {NULL}  /* Sentinel */
};


static PyTypeObject Progress_Type = {
   PyObject_HEAD_INIT(NULL)
   0,                         /*ob_size*/
   "daqpp.Progress",             /*tp_name*/
   sizeof(daqpp_Progress),       /*tp_basicsize*/
   0,                         /*tp_itemsize*/
   (destructor)progress_dealloc,                         /*tp_dealloc*/
   0,                         /*tp_print*/
   0,                         /*tp_getattr*/
   0,                         /*tp_setattr*/
   0,                         /*tp_compare*/
   (reprfunc)0,                         /*tp_repr*/
   0,                         /*tp_as_number*/
   0,                         /*tp_as_sequence*/
   0,                         /*tp_as_mapping*/
   0,                         /*tp_hash */
   (ternaryfunc)0,                         /*tp_call*/
   (reprfunc)0,                 /*tp_str*/
   0,                         /*tp_getattro*/
   0,                         /*tp_setattro*/
   0,                         /*tp_as_buffer*/
   Py_TPFLAGS_DEFAULT| Py_TPFLAGS_BASETYPE,        /*tp_flags*/
   "Progress wrapper",           /* tp_doc */
       0,		               /* tp_traverse */
    0,		               /* tp_clear */
    0,		               /* tp_richcompare */
    0,		               /* tp_weaklistoffset */
    0,		               /* tp_iter */
    0,		               /* tp_iternext */
    progress_methods,             /* tp_methods */
    progress_members,             /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)progress_init,      /* tp_init */
    0,                         /* tp_alloc */
    progress_new,                 /* tp_new */
};
void register_progress(PyObject *dict) {
   daqpp_register_class(dict,"Progress",
			&Progress_Type,
			0);
   daqpp_add_int_constant(dict,"show_nothing", Progress::show_nothing);
   daqpp_add_int_constant(dict,"show_status", Progress::show_status);
   daqpp_add_int_constant(dict,"show_time", Progress::show_time);
   daqpp_add_int_constant(dict,"show_average", Progress::show_time); 
   daqpp_add_int_constant(dict,"show_left", Progress::show_left); 
   daqpp_add_int_constant(dict,"show_memory", Progress::show_memory);
}
