/* -*- mode: c++ -*- */
#ifndef __pydaqpp_base_h__
#define __pydaqpp_base_h__
#include <Python.h>
#include <pythread.h>
#include <structmember.h>
#include <string>
#include <iostream>
#include <iomanip>
#include <map>
#include <DAQ++/Thread.h>
extern PyInterpreterState *daqpp_interp;

__BEGIN_DECLS

#if (PY_VERSION_HEX < 0x02050000)
typedef int Py_ssize_t;
typedef inquiry lenfunc;
typedef intargfunc ssizeargfunc;
typedef intintargfunc ssizessizeargfunc;
typedef intobjargproc ssizeobjargproc;
typedef intintobjargproc ssizessizeobjargproc;
typedef getreadbufferproc readbufferproc;
typedef getwritebufferproc writebufferproc;
typedef getsegcountproc segcountproc;
#endif

#if (PY_VERSION_HEX <= 0x02040400)
typedef char * pycharp_t;
#define Py_DecRef Py_DECREF
#define Py_IncRef Py_INCREF
#if (PY_VERSION_HEX <= 0x02040300)
inline bool PyEval_ThreadsInitialized() { return false; }
#endif
#else
typedef const char * pycharp_t;
#endif
void daqpp_register_class(PyObject *dict,
                          const char *type_name,
                          PyTypeObject *type,
                          PyObject *bases);
void daqpp_add_int_constant(PyObject *dict,
                            const char *name,
                            int value);
void register_daqid(PyObject *dict);
void register_module(PyObject *dict);
void register_runmanager(PyObject *dict);
void register_daqmanager(PyObject *dict);
void register_runstate(PyObject *dict);
void register_progress(PyObject *dict);
void register_monitor(PyObject *dict);
void register_daqpp_buffer(PyObject *dict);
void register_fifo(PyObject *dict);
void register_gtimer(PyObject *dict);
void register_observer(PyObject *dict);
void register_observable(PyObject *dict);
void register_paramholder(PyObject *dict);
void register_fileIOManager(PyObject *dict);
PyObject *PyDAQpp_DAQid_New(const char *);
PyObject *PyDAQpp_Module_New(void *);
PyObject *PyDAQpp_RunManager_New(void *);
PyObject *PyDAQpp_RunState_New();
PyObject *PyDAQpp_Buffer_New(char c,int sz, char *);
PyObject *PyDAQpp_Monitor_New(void *);
PyObject *PyDAQpp_Observable_New(void *);

#ifndef PyAPI_FUNC
#define PyAPI_FUNC(type) type
#endif

PyAPI_FUNC(int) PyDAQpp_Module_Check(PyObject *);
PyAPI_FUNC(int) PyDAQpp_RunManager_Check(PyObject *);
PyAPI_FUNC(int) PyDAQpp_Buffer_Check(PyObject *O);
PyAPI_FUNC(int) PyDAQpp_DAQid_Check(PyObject *O);

extern PyObject *DAQpp_Error;

__END_DECLS

/**
 * This a wrapper for DAQ++ objects within the Python environment
 */
class PyDAQpp
{
    private:
        PyObject *_self;
        unsigned long flags;

    protected:
        int get_rc(PyObject *obj, const char *name);
        int call_int_method(pycharp_t name, pycharp_t fmt, ...);
        void call_void_method(pycharp_t name, pycharp_t fmt, ...);
        PyObject *call_obj_method(pycharp_t name, pycharp_t fmt, ...);
        static DAQpp::Mutex mtx;

    public:
        PyDAQpp() : _self(0), flags(0)
        {
        }

        virtual ~PyDAQpp();

        PyObject *self()
        {
            return _self;
        }

        bool has_object()
        {
            return (_self!=0);
        }

        bool python_call()
        {
            return flags&1L;
        }

        bool external_call()
        {
            return flags&2L;
        }

        void set_python_call()
        {
            flags |= 1L;
        }

        void clear_python_call()
        {
            flags &= ~1L;
        }

        void set_external_call()
        {
            flags |=2L;
        }

        void clear_external_call()
        {
            flags &=~2L;
        }

        void set_object(PyObject *o);

        static DAQpp::Mutex &lock()
        {
            return mtx;
        }

        virtual int check_attributes()
        {
            return 0;
        }
};

namespace DAQpp
{
    class Module;
    class RunManager;
}

DAQpp::Module *get_the_module(PyObject *o);
DAQpp::RunManager *get_the_runmanager(PyObject *o);
void daqpp_observer_start();
void daqpp_observer_stop();
void daqpp_print_error();

extern PyObject *DAQpp_Error;


#ifndef PyDoc_STRVAR
#define PyDoc_VAR(name) static char name[]
#define PyDoc_STRVAR(name,str) PyDoc_VAR(name) = PyDoc_STR(str)
#define PyDoc_STR(str) str
#endif

#endif
