#include "pydaqpp.h"
#include "pydaqpp_debug.h"
#include <iostream>
#include <iomanip>
#include <sstream>
#include "pydaqpp_importSequence.h"
#include "pydaqpp_buffer.h"

#if PY_VERSION_HEX < 0x2030000
static PyObject *
PyObject_SelfIter(PyObject *obj)
{
    Py_INCREF(obj);
    return obj;
}
#endif


// The getters and setter for the different types
static PyObject *c_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyString_FromStringAndSize(&((char *)b->item)[i], 1);
}
static int c_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    char x;
    if (!PyArg_Parse(v, "c;array must be char", &x))
        return -1;
    if (i>=0)
        ((char *)b->item)[i]=x;
    return 0;
}

static PyObject *b_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    long x = ((char *)b->item)[i];
    if (x>=128)
        x -= 128;
    return PyInt_FromLong(x);
}

static int b_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    char x;
    if (!PyArg_Parse(v, "b;array must be integer", &x))
        return -1;
    if (i>=0)
        ((char *)b->item)[i]=x;
    return 0;
}

static PyObject *B_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    long x = ((unsigned char *)b->item)[i];
    return PyInt_FromLong(x);
}

#define B_setitem b_setitem

static PyObject *h_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyInt_FromLong((long) ((short *)b->item)[i]);
}

static int h_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    short x;
    if (!PyArg_Parse(v, "h;array item must be integer", &x))
        return -1;
    if (i >= 0)
        ((short *)b->item)[i] = x;
    return 0;
}

static PyObject *H_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyInt_FromLong((long) ((unsigned short *)b->item)[i]);
}

#define H_setitem h_setitem

static PyObject *i_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyInt_FromLong((long) ((int *)b->item)[i]);
}

static int i_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    int x;
    if (!PyArg_Parse(v, "i;array item must be integer", &x))
        return -1;
    if (i >= 0)
        ((int *)b->item)[i] = x;
    return 0;
}

static PyObject *I_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyLong_FromUnsignedLong((unsigned long) ((unsigned int *)b->item)[i]);
}

static int I_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    unsigned long x;
    if (PyLong_Check(v))
    {
        x = PyLong_AsUnsignedLong(v);
        if (x == (unsigned long) -1 && PyErr_Occurred())
            return -1;
    }
    else
    {
        if (!PyArg_Parse(v, "l;array item must be integer", &x))
            return -1;
    }
    if (i >= 0)
        ((unsigned int *)b->item)[i] = x;
    return 0;
}

static PyObject *l_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyInt_FromLong(((long *)b->item)[i]);
}

static int l_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    long x;
    if (!PyArg_Parse(v, "l;array item must be integer", &x))
        return -1;
    if (i >= 0)
        ((long *)b->item)[i] = x;
    return 0;
}

static PyObject *L_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyLong_FromLong(((long *)b->item)[i]);
}

static int L_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    unsigned long x;
    if (PyLong_Check(v))
    {
        x = PyLong_AsUnsignedLong(v);
        if (x == (unsigned long) -1 && PyErr_Occurred())
            return -1;
    }
    else
    {
        if (!PyArg_Parse(v, "l;array item must be integer", &x))
            return -1;
    }
    if (i >= 0)
        ((unsigned long *)b->item)[i] = x;
    return 0;
}

static PyObject *f_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyFloat_FromDouble((double) ((float *)b->item)[i]);
}

static int f_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    float x;
    if (!PyArg_Parse(v, "f;array item must be float", &x))
        return -1;
    if (i >= 0)
        ((float *)b->item)[i] = x;
    return 0;
}

static PyObject *d_getitem(daqpp_buffer *b, Py_ssize_t i)
{
    return PyFloat_FromDouble(((double *)b->item)[i]);
}

static int d_setitem(daqpp_buffer *b, Py_ssize_t i, PyObject *v)
{
    double x;
    if (!PyArg_Parse(v, "d;array item must be float", &x))
        return -1;
    if (i >= 0)
        ((double *)b->item)[i] = x;
    return 0;
}

/* Description of types */
static struct daqpp_buffer_descriptor descriptors[] = {
	{'c', sizeof(char),   c_getitem, c_setitem},
	{'b', sizeof(char),   b_getitem, b_setitem},
	{'B', sizeof(unsigned char),   B_getitem, B_setitem},
	{'h', sizeof(short),  h_getitem, h_setitem},
	{'H', sizeof(unsigned short),  H_getitem, H_setitem},
	{'i', sizeof(int),    i_getitem, i_setitem},
	{'I', sizeof(unsigned int),    I_getitem, I_setitem},
	{'l', sizeof(long),   l_getitem, l_setitem},
	{'L', sizeof(unsigned long),   L_getitem, L_setitem},
	{'f', sizeof(float),  f_getitem, f_setitem},
	{'d', sizeof(double), d_getitem, d_setitem},
	{'\0', 0, 0, 0}
};


static daqpp_buffer_descriptor *find_descriptor(char x)
{
    struct daqpp_buffer_descriptor *d = descriptors;
    for (; d->type!='\0'; d++)
    {
        if (d->type==x)
            return d;
    }
    return NULL;
}


static void daqpp_buffer_destructor(void *ptr)
{
    PyMem_Free(ptr);
}


static PyObject *daqpp_buffer_new_object(daqpp_buffer *self, char c, int sz, char *ptr)
{
    daqpp_buffer_descriptor *descr=NULL;
    descr = find_descriptor(c);
    if (!descr)
    {
        std::ostringstream ostr;
        ostr << "daqpp.Buffer: invalid data type "<< c << std::endl << "\t(must be c, b, B, u, h, H, i, I, l, L, f or d)";
        PyErr_SetString(DAQpp_Error, ostr.str().c_str());
        return NULL;
    }

    self->descr=descr;
    self->len=sz;
    self->item=ptr;
    self->nitems = sz/descr->size;
    self->weakreflist = NULL;
    if (self->own && self->item)
    {
        self->cobj = PyCObject_FromVoidPtr((void *)ptr,daqpp_buffer_destructor);
    }
    else
        self->cobj = 0;
    return (PyObject *)self;
}

static PyObject *daqpp_buffer_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    daqpp_buffer *self;
    WHOAMI;
    self = (daqpp_buffer *)type->tp_alloc(type, 0);
    if (!self)
        return (PyObject *)0;

    if (self!=NULL)
    {
        self->own = 0;
        self->nitems=0;
        self->len =0;
        self->item =0;
        self->descr =0;
        self->weakreflist=0;
        self->cobj = 0;
        self->dict = PyDict_New();
    }
    return (PyObject *)self;
}

static int daqpp_buffer_init(daqpp_buffer *self, PyObject *args, PyObject *kwds)
{
    char c;
    PyObject *in=NULL;

    // Check that we do not get any keyword argument
    if (kwds != NULL)
    {
        int i= PyObject_Length(kwds);
        if (i < 0)
            return -1;
        else if (i > 0)
        {
            PyErr_SetString(DAQpp_Error, "daqpp.Buffer constructor takes "
                            "no keyword arguments");
            return -1;
        }
    }

    // Get the type and the Object
    if (!PyArg_ParseTuple(args, "cO:daqpp.Buffer.__init__", &c, &in))
        return -1;

    if (in==NULL)
    {
        PyErr_SetString(DAQpp_Error, "daqpp.Buffer is getting a NULL object to clone");
        return -1;
    }

    // Get the size and the memory ptr of the buffer
    Py_ssize_t sz=0;
    char *ptr=0;
    if (PyObject_AsWriteBuffer(in, (void **)&ptr, &sz) )
    {
        // It is not a buffer type, so we try with a sequence, but now
        // we are not able to map the memory of that object
        if (PySequence_Check(in) )
        {
            PyErr_Clear(); // this is set by PyObject_AsWriteBuffer
            try
            {
                ptr = import_sequence(in,sz,c);
            }
            catch ( SeqException e )
            {
                std::ostringstream ostr;
                ostr << "daqpp.Buffer: " << e.what();
                PyErr_SetString(DAQpp_Error,ostr.str().c_str());
                if (ptr)
                    PyMem_Free(ptr);
                return -1;
            }
            self->own = 1;
        }
        // Then we just have the size of the buffer
        else if ( PyInt_Check(in) )
        {
            PyErr_Clear(); // this is set by PyObject_AsWriteBuffer
            daqpp_buffer_descriptor *descr = find_descriptor(c);
            sz = PyInt_AsLong(in)*descr->size;
            ptr = (char *)PyMem_Malloc(sz);
            self->own = 1;
        }
        else
        {
            PyErr_SetString(PyExc_ValueError, "daqpp.Buffer: import object is neither a buffer nor a sequence");
            return -1;
        }
    }
    else
        self->own = 0;

    if (!sz)
        return -1;


    self = (daqpp_buffer *)daqpp_buffer_new_object(self, c, sz, ptr);
    return 0;
}


static void daqpp_buffer_dealloc(daqpp_buffer* self)
{
    WHOAMI;
    if (self->cobj)
    {
        Py_DECREF(self->cobj);
    }
    if (self->weakreflist != NULL)
        PyObject_ClearWeakRefs((PyObject *) self);

    Py_DECREF(self->dict);
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject *daqpp_buffer_tostring(daqpp_buffer *self, PyObject *unused)
{
    return PyString_FromStringAndSize(self->item, self->len);
}

static PyObject *daqpp_buffer_get_item(daqpp_buffer *self, int i)
{
    PyObject *out=NULL;
    if (i>=0&& i<self->nitems)
    {
        out = (*self->descr->getitem)(self, i);
    }
    if (out==NULL)
    {
        PyErr_SetString(PyExc_IndexError, "daqpp.Buffer __getitem__ index out of range");
    }
    return out;
}

static int daqpp_buffer_length(daqpp_buffer *self)
{
    return self->nitems;
}

static PyObject *daqpp_buffer_concat(daqpp_buffer *self, PyObject *o)
{
    Py_INCREF(self);
    return (PyObject *)self;
}

static PyObject *daqpp_buffer_slice(daqpp_buffer *self, Py_ssize_t ilow, Py_ssize_t ihigh)
{
    daqpp_buffer *out;
    int item_size = self->descr->size;
    int size;
    if (ilow<0)
        ilow=0;
    else if (ilow>self->nitems)
        ilow = self->nitems;

    if (ihigh<0)
    {
        ihigh=0;
    }
    else if (ihigh<ilow)
    {
        ihigh=ilow;
    }
    if (ihigh>self->nitems)
    {
        ihigh=self->nitems;
    }
    size = ihigh-ilow;

    out = (daqpp_buffer *)PyDAQpp_Buffer_New((char)self->descr->type,
                                             size*item_size,
                                             self->item + ilow*item_size
                                             );
    if (self->cobj)
    {
        out->cobj = self->cobj;
        Py_INCREF(out->cobj);
    }
   return (PyObject *)out;
}

static int daqpp_buffer_ass_item(daqpp_buffer *self, int i, PyObject *v)
{
    if (i<0|| i>=self->nitems)
    {
        PyErr_SetString(PyExc_IndexError, "daqpp.Buffer assignment index out of range");
        return -1;
    }
    if (v==NULL)
    {
        PyErr_SetString(PyExc_ValueError, "daqpp.Buffer assignment gets a NULL object");
        return -1;
    }
    return (*self->descr->setitem)(self, i, v);
}

static int daqpp_buffer_ass_slice(daqpp_buffer *self, int ilow, int ihigh, PyObject *v)
{
    PyErr_SetString(PyExc_SystemError,"daqpp.Buffer: slice assingment not implemented");
    return -1;
}

static PySequenceMethods daqpp_buffer_as_sequence = {
   (lenfunc)daqpp_buffer_length,           /*sq_length*/
   (binaryfunc)daqpp_buffer_concat,        /*sq_concat*/
   (ssizeargfunc)0,		           /*sq_repeat*/
   (ssizeargfunc)daqpp_buffer_get_item,	   /*sq_item*/
   (ssizessizeargfunc)daqpp_buffer_slice,	   /*sq_slice*/
   (ssizeobjargproc)daqpp_buffer_ass_item,   /*sq_ass_item*/
   (ssizessizeobjargproc)daqpp_buffer_ass_slice,/*sq_ass_slice*/
   (objobjproc)0,		           /*sq_contains*/
   (binaryfunc)0,	                   /*sq_inplace_concat*/
   (ssizeargfunc)0	                   /*sq_inplace_repeat*/
};

static int daqpp_buffer_getreadbuf(daqpp_buffer *self, int index, const void **ptr)
{
    if (index != 0)
    {
        PyErr_SetString(PyExc_SystemError, "Accessing non-existent array segment");
        return -1;
    }
    *ptr = (void *)self->item;
    return self->len;
}

static int daqpp_buffer_getwritebuf(daqpp_buffer *self, int index, const void **ptr)
{
    if (index != 0)
    {
        PyErr_SetString(PyExc_SystemError, "Accessing non-existent array segment");
        return -1;
    }
    *ptr = (void *)self->item;
    return self->len;
}

static int daqpp_buffer_getsegcount(daqpp_buffer *self, int *lenp)
{
    if (lenp )
        *lenp = self->len;
    return 1;
}


static PyBufferProcs daqpp_buffer_as_buffer = {
	(readbufferproc)daqpp_buffer_getreadbuf,
	(writebufferproc)daqpp_buffer_getwritebuf,
	(segcountproc)daqpp_buffer_getsegcount,
};

static PyObject *daqpp_buffer_get_type(daqpp_buffer *self, void *closure)
{
    char tc = self->descr->type;
    return PyString_FromStringAndSize(&tc, 1);
}

static int daqpp_buffer_set_type(daqpp_buffer *self, PyObject *value, void *context)
{
    daqpp_buffer_descriptor *descr=NULL;
    char *c=PyString_AsString(value);
    if (c)
        descr = find_descriptor(*c);

    if (!descr)
    {
        std::ostringstream ostr;
        ostr << "daqpp.Buffer: invalid data type "<< c << std::endl<< "\t(must be c, b, B, u, h, H, i, I, l, L, f or d)";
        PyErr_SetString(DAQpp_Error, ostr.str().c_str());
        return -1;
    }

    if (self->len < self->descr->size)
    {
        std::ostringstream ostr;
        ostr << "Requested type ['"<< c << "'] "<< "is bigger thant the buffer size";
        PyErr_SetString(DAQpp_Error, ostr.str().c_str());
        return -1;
    }
    self->descr=descr;
    self->nitems=self->len/self->descr->size;

    return 0;
}

static PyObject *daqpp_buffer_get_itemsize(daqpp_buffer *self, void *closure)
{
    return PyInt_FromLong((long)self->descr->size);
}

static PyObject *daqpp_buffer_get_nitems(daqpp_buffer *self, void *closure)
{
    return PyInt_FromLong((long)self->nitems);
}

static PyObject *daqpp_buffer_get_size(daqpp_buffer *self, void *closure)
{
    return PyInt_FromLong((long)self->len);
}

static PyGetSetDef daqpp_buffer_getsets [] = {
   {"type",(getter)daqpp_buffer_get_type,(setter)daqpp_buffer_set_type,"type code used for the buffer"},
   {"itemsize",(getter)daqpp_buffer_get_itemsize,NULL,"the size, in bytes, of one buffer item"},
   {"nitems",(getter)daqpp_buffer_get_nitems,NULL,"the number of items in the array"},
   {"size",(getter)daqpp_buffer_get_size,NULL,"the size, in bytes, of the buffer"},
   {NULL}
};

static PyObject *daqpp_buffer_info(daqpp_buffer *self, PyObject *unused)
{
    PyObject* retval= NULL;
    retval = PyTuple_New(2);
    if (!retval)
        return NULL;

    PyTuple_SET_ITEM(retval, 0, PyLong_FromVoidPtr(self->item));
    PyTuple_SET_ITEM(retval, 1, PyInt_FromLong((long)(self->nitems)));

    return retval;
}

static PyObject *daqpp_buffer_tolist(daqpp_buffer* self, PyObject *unused)
{
    PyObject *list = PyList_New(self->nitems);
    Py_ssize_t i;

    if (list==NULL)
        return NULL;

    for (i=0; i<self->nitems; i++)
    {
        PyObject *v = daqpp_buffer_get_item(self, i);
        if (v==NULL)
        {
            Py_DECREF(list);
            return NULL;
        }
        PyList_SetItem(list, i, v);
    }
    return list;
}

static PyObject *daqpp_buffer_repr(daqpp_buffer *a)
{
    char buf[256], typecode;
    PyObject *s, *t, *v= NULL;
    int len;

    len = a->nitems;
    typecode = a->descr->type;
    if (len == 0)
    {
        PyOS_snprintf(buf, sizeof(buf), "Buffer('%c')", typecode);
        return PyString_FromString(buf);
    }

    if (typecode == 'c')
        v = daqpp_buffer_tostring(a, NULL);
    else
        v = daqpp_buffer_tolist(a, NULL);

    t = PyObject_Repr(v);
    Py_XDECREF(v);

    PyOS_snprintf(buf, sizeof(buf), "Buffer('%c', ", typecode);
    s = PyString_FromString(buf);
    PyString_ConcatAndDel(&s, t);
    PyString_ConcatAndDel(&s, PyString_FromString(")"));
    return s;
}


PyMethodDef daqpp_buffer_methods[] = {
   {"buffer_info", (PyCFunction)daqpp_buffer_info, METH_NOARGS,"."},
   {"tolist",	(PyCFunction)daqpp_buffer_tolist, METH_NOARGS,"."},
   {"tostring", (PyCFunction)daqpp_buffer_tostring, METH_NOARGS, "." },
   {NULL,		NULL}		/* sentinel */
};
static PyObject *daqpp_buffer_iter(daqpp_buffer *self);


static PyTypeObject DBuffertype = {
	PyObject_HEAD_INIT(NULL)
	0,
	"daqpp.Buffer",
	sizeof(daqpp_buffer),
	0,
	(destructor)daqpp_buffer_dealloc,		/* tp_dealloc */
	0,					/* tp_print */
	0,					/* tp_getattr */
	0,					/* tp_setattr */
	0,					/* tp_compare */
	(reprfunc)daqpp_buffer_repr,			/* tp_repr */
	0,					/* tp_as _number*/
	&daqpp_buffer_as_sequence,			/* tp_as _sequence*/
	0,			/* tp_as _mapping*/
	0, 					/* tp_hash */
	0,					/* tp_call */
	0,					/* tp_str */
	PyObject_GenericGetAttr,		/* tp_getattro */
	0,					/* tp_setattro */
	&daqpp_buffer_as_buffer,			/* tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE| Py_TPFLAGS_HAVE_WEAKREFS,  /* tp_flags */
	"Memory array wrapper",				/* tp_doc */
 	0,					/* tp_traverse */
	0,					/* tp_clear */
	0,			/* tp_richcompare */
	offsetof(daqpp_buffer, weakreflist),	/* tp_weaklistoffset */
	(getiterfunc)daqpp_buffer_iter,		/* tp_iter */
	0,					/* tp_iternext */
	daqpp_buffer_methods,				/* tp_methods */
	0,					/* tp_members */
	daqpp_buffer_getsets,				/* tp_getset */
	0,					/* tp_base */
	0,					/* tp_dict */
	0,					/* tp_descr_get */
	0,					/* tp_descr_set */
	offsetof(daqpp_buffer, dict),					/* tp_dictoffset */
	(initproc)daqpp_buffer_init,                      /* tp_init */
	PyType_GenericAlloc,			/* tp_alloc */
	daqpp_buffer_new,				/* tp_new */
};

/*-----------------------------------------------------------
 * The Iterator
 *-----------------------------------------------------------*/

typedef struct
{
    PyObject_HEAD
    long index;
    daqpp_buffer *buffer;
    PyObject * (*getitem)(struct daqpp_buffer *, Py_ssize_t);
} daqpp_buffer_iter_object;

static PyObject *daqpp_buffer_iter_next(daqpp_buffer_iter_object *it)
{
    //	assert(PyArrayIter_Check(it));
    if (it->index < it->buffer->nitems)
        return (*it->getitem)(it->buffer, it->index++);
    return NULL;
}

static void daqpp_buffer_iter_dealloc(daqpp_buffer_iter_object *it)
{
    PyObject_GC_UnTrack(it);
    Py_XDECREF(it->buffer);
    PyObject_GC_Del(it);
}

static int daqpp_buffer_iter_traverse(daqpp_buffer_iter_object *it, visitproc visit, void *arg)
{
    if (it->buffer != NULL)
        return visit((PyObject *)(it->buffer), arg);
    return 0;
}

static PyTypeObject PyDAQppBufferIter_Type = {
   PyObject_HEAD_INIT(NULL)
   0,                                      /* ob_size */
   "bufferiterator",                        /* tp_name */
   sizeof(daqpp_buffer_iter_object),      /* tp_basicsize */
   0,                                      /* tp_itemsize */
   /* methods */
   (destructor)daqpp_buffer_iter_dealloc,		/* tp_dealloc */
   0,                                      /* tp_print */
   0,                                      /* tp_getattr */
   0,                                      /* tp_setattr */
   0,                                      /* tp_compare */
   0,                                      /* tp_repr */
   0,                                      /* tp_as_number */
   0,                                      /* tp_as_sequence */
   0,                                      /* tp_as_mapping */
   0,                                      /* tp_hash */
   0,                                      /* tp_call */
   0,                                      /* tp_str */
   PyObject_GenericGetAttr,                /* tp_getattro */
   0,                                      /* tp_setattro */
   0,                                      /* tp_as_buffer */
   Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,/* tp_flags */
   0,                                      /* tp_doc */
   (traverseproc)daqpp_buffer_iter_traverse,	/* tp_traverse */
   0,					/* tp_clear */
   0,                                      /* tp_richcompare */
   0,                                      /* tp_weaklistoffset */
   PyObject_SelfIter,			/* tp_iter */
   (iternextfunc)daqpp_buffer_iter_next,		/* tp_iternext */
   0,					/* tp_methods */
};

static PyObject *daqpp_buffer_iter(daqpp_buffer *self)
{
    daqpp_buffer_iter_object *it;
    it = PyObject_GC_New(daqpp_buffer_iter_object, &PyDAQppBufferIter_Type);
    if (it == NULL)
        return NULL;

    Py_INCREF(self);
    it->buffer = self;
    it->index = 0;
    it->getitem = self->descr->getitem;
    PyObject_GC_Track(it);
    return (PyObject *)it;
}

void register_daqpp_buffer(PyObject *dict)
{
    //    DBuffertype.ob_type = &PyType_Type;
    //    PyDAQppBufferIter_Type.ob_type = &PyType_Type;
    daqpp_register_class(dict, "Buffer", &DBuffertype, 0);
}

int PyDAQpp_Buffer_Check(PyObject *O)
{
    return PyType_IsSubtype(O->ob_type, &DBuffertype);
}

PyObject *PyDAQpp_Buffer_New(char c, int sz, char *ptr)
{
    int own=0;
    if (ptr==0&& sz>0)
    {
        own=1;
        ptr = (char *)PyMem_Malloc(sz);
        if (ptr==NULL)
        {
            return PyErr_NoMemory();
        }
    }
    daqpp_buffer *self= PyObject_New(daqpp_buffer,&DBuffertype);
    self->own = own;
    self->dict = PyDict_New();
    daqpp_buffer_new_object(self, c, sz, ptr);
    return (PyObject *)self;
}
