#include "pydaqpp.h"
#include <iostream>
#include <sstream>
#include "pydaqpp_importSequence.h"

class SqItem
{
    private:
        union Value
        {
            char c;
            long l;
            double f;
        };
        Value val;
        int size;
        PyTypeObject *type;
        char ctype;
    public:
        SqItem(PyObject *o) :
            size(0), type(0), ctype(0)
            {
            if (o)
                set_type(o);
            }
        SqItem(char c=0) :
            size(0), type(0), ctype(c)
            {
            if (c)
                set_type(c);
            }
        void set_type(PyObject *o);
        void set_type(char c);
        bool good()
        {
            return type!=0;
        }
        int get_size() const
        {
            return size;
        }
        int append(char *ptr, PyObject *o) throw(SeqException);
};

void SqItem::set_type(PyObject *first)
{
    if ( PyInt_CheckExact(first))
    {
        ctype='i';
        type = &PyInt_Type;
        size = sizeof(long);
    }
    else if ( PyFloat_CheckExact(first))
    {
        ctype='f';
        type = &PyFloat_Type;
        size = sizeof(double);
    }
    else if ( PyString_Check(first))
    {
        ctype = 's';
        type= &PyString_Type;
        size = sizeof(char);
    }
}

void SqItem::set_type(char c)
{
    switch (c)
    {
        case 'b':
        case 'B':
        case 'u':
        case 'h':
        case 'H':
        case 'I':
        case 'i':
            ctype='i';
            type = &PyInt_Type;
            size = sizeof(long);
            break;
        case 'f':
        case 'd':
            ctype='f';
            type = &PyFloat_Type;
            size = sizeof(double);
            break;
        case 's':
        case 'c':
            ctype = 's';
            type= &PyString_Type;
            size = sizeof(char);
            break;
        case 'l':
        case 'L':
            ctype='i';
            type = &PyLong_Type;
            size = sizeof(unsigned long);
            break;
        default:
        {
            std::ostringstream ostr;
            std::cout << "Invalid type '"<< c << "'. Setting 's'"<< std::endl;
            ctype = 's';
            type= &PyString_Type;
            size = sizeof(char);
        }
    }
}

int SqItem::append(char *ptr, PyObject *obj) throw(SeqException)
{
    PyObject *res;
    if ( !type )
        throw SeqException("Undefined type");

    if (obj==NULL)
        throw SeqException("truncated sequence");

    switch (ctype)
    {
        case 'i':
            if ( !(res = PyNumber_Int(obj)))
                throw SeqException("Inconsistent item type");
            val.l = PyInt_AsLong(res);
            Py_DECREF(res);
            break;
        case 'f':
            if ( !(res = PyNumber_Float(obj)))
                throw SeqException("Inconsistent item type");
            val.f = PyFloat_AsDouble(res);
            Py_DECREF(res);
            break;
        case 's':
            val.c = *PyString_AsString(obj);
            break;
    }
    memcpy(ptr, &val, size);
    return size;
}

char *import_sequence(PyObject *obj, Py_ssize_t &size, char tp)
throw(SeqException)
{
    Py_ssize_t i, len;
    PyObject *item=0;
    char *ptr;

    size=0;
    if (!PySequence_Check(obj))
        throw SeqException("import_sequence: object is not a Sequence");

    if ( (len = PySequence_Length(obj))<=0)
        throw SeqException("import_sequence: Empty sequence");



    SqItem st;
    if (tp )
        st.set_type(tp);
    else
    {
        item = PySequence_GetItem(obj, 0);
        st.set_type(item );
        Py_DECREF(item);
    }

    if (!st.good())
    {
        std::ostringstream ostr;
        ostr << "Invalid type "<< item->ob_type->tp_name;
        throw SeqException(ostr.str());
    }

    size = len*st.get_size();
    ptr = (char *)PyMem_Malloc(size);
    if (!ptr)
        throw SeqException("No memory");

    char *vptr=ptr;
    try
    {
        for (i=0;i<len;i++)
        {
            try
            {
                item = PySequence_GetItem(obj,i);
                vptr += st.append(vptr,item);
                Py_DECREF(item);
            }
            catch ( ... )
            {
                Py_DECREF(item);
                throw;
            }
        }
    }
    catch ( ... )
    {
        if ( PyErr_Occurred() )
        {
            daqpp_print_error();
            PyErr_Print();
        }
        PyMem_Free(ptr);
        size=0;
        ptr=0;
        throw;
    }
    return ptr;
}
