#/usr/bin/env python
""" A simple implementation of a histogram
"""
import struct
import cStringIO
import struct

class Histogram(object):
    """ A Histogram
    """
    def __init__(self, name, title, nbin, xmin, xmax):
        self.name = name
        self.title = title
        self.nbin = nbin
        self.xmin = xmin
        self.xmax = xmax
        self.range = xmax-xmin
        self.step = self.range/float(nbin)
        self.hsum = 0.0
        self.tsum = 0.0
        self.data = [ 0.0 for i in range(0, nbin)]

    def bin_center(self, bin):
        return self.xmin + (0.5+self.nbin)*self.step

    def bin(self, x):
        return int( self.nbin * (x-self.xmin)/self.range )

    def fill(self, x, w=1.0):
        ibin = self.bin(x)
        self.tsum += w
        if ibin>=0 and ibin < self.nbin:
            self.data[ibin] += w
            self.hsum += w

    def write(self):
        """ Returns a string that can be saved in a file
        """
        out = cStringIO.StringIO()
        out.write( struct.pack("i", self.nbin) )
        out.write( struct.pack("5d", self.xmin, self.xmax, self.hsum, self.tsum, self.step) )
        out.write( struct.pack("%dd" % self.nbin, *self.data))
        sz = len(self.name)
        out.write( struct.pack("i", sz))
        out.write( struct.pack("%dc" % sz, *self.name))
        sz = len(self.title)
        out.write( struct.pack("i", sz))
        out.write( struct.pack("%dc" % sz, *self.title))
        ss = out.getvalue()
        out.close()
        return ss

    def read(self, val):
        """ reads a histogram from memory
        """
        
        offs = 0
        sz = struct.calcsize("i")
        self.nbin = struct.unpack("i",val[offs:offs+sz])[0]
        offs += sz
        
        sz = struct.calcsize("5d")
        self.xmin, self.xmax, self.hsum, self.tsum, self.step = struct.unpack("5d", val[offs:offs+sz])
        offs += sz
        
        fmt = "%dd" % self.nbin
        sz = struct.calcsize(fmt)
        self.data = struct.unpack(fmt, val[offs:offs+sz])
        offs += sz

        sz = struct.calcsize("i")
        vv = struct.unpack("i", val[offs:offs+sz])
        offs += sz
        self.name = val[offs:offs+vv[0]]

        offs += vv[0]
        sz = struct.calcsize("i")
        vv = struct.unpack("i", val[offs:offs+sz])
        offs += sz
        self.title = val[offs:offs+vv[0]]

    def dump(self):
        """  Prints the contents
        """
        print "Histogram:", self.name, '[', self.title,']'
        print self.nbin, self.xmin, self.xmax, 'step', self.step
        for i in range(0, self.nbin):
            if i>0 and not (i%6):
                print ''
            print "%12.1f" % self.data[i],

        print ''




