#!/usr/bin/env python
"""
MsgManager.py

 Created on: Jan 11, 2011
     Author: lacasta
     
     
 This script shows an example of an Observer (MsgManager) receiving messages
 from the DAQmanager. When a new Module or RunManager is created, it 
 automatically registers an Observer of the Parameters and, in the case of
 the RunManager, to its RunState
   
"""

import daqpp

class ParamObserver(object):
    """ This will 'observe' the Parameters of a DAQObject
        The input argument is the name of the DAQObject
    """
    def __init__(self, name):
        self.name = name
    
    def update(self, val):
        """ This will be called whenever a Parameter changes
        """
        print self.name, "parameter", val

class StateObserver(object):
    """ This will 'observe' the RunState of a RunManager.
    """
    def __init__(self, name):
        self.name = name
        
    def update(self, val):
        """ Called whenever the RunState changes status
        """
        print "Changing state [%s]" % self.name, val

class MsgManager(object):
    """ This is the Object that registers to get notifications
        from the DAQmanager.
    """
    def update(self, val):
        """ The callback method
        """
        
        msg = val[0]
        name = val[1]
        if msg == daqpp.startDAQ:
            print "*** Starting DAQ"
        elif msg == daqpp.stopDAQ:
            print "*** Stopping DAQ"
        elif msg == daqpp.nwModule:
            print "*** adding module", name
            mod = daqpp.find_module(name)
            if mod:
                mod.addObserver(ParamObserver(name))
                
        elif msg == daqpp.rmModule:
            print "*** removing", name
        elif msg == daqpp.nwRunmanager:
            print "*** adding run manager", name
            rm = daqpp.find_runmanager(name)
            if rm:
                rm.addObserver(param=ParamObserver(name), state=StateObserver(name))
        elif msg == daqpp.rmRunManager:
            print "*** removing run manger", name
        
