#include <iostream>
#include <vector>
#include <cmath>
#include <DAQ++/DAQmanager.h>
#include "MyModule.h"

static double randflat(double a,double b) {
   double x = ((double)rand())/((double)RAND_MAX);
   return a+x*(b-a);
}
static double randuniform() {
   double x = ((double)rand())/((double)RAND_MAX);
   return x;
}
static double randgauss(double mean,double stdev) {
   static int haveCachedNormal = 0;
   static double  cachedNormal = 0.;
   static double  pMean=0.,pStdDev=0.;

   if ( mean!=pMean || pStdDev!=stdev ) {
      haveCachedNormal = 0;
      pMean = mean;
      pStdDev=stdev;
   }
   if (haveCachedNormal == 1) {
      haveCachedNormal = 0;
      return(cachedNormal * pStdDev + pMean );
   } else {
      for(;;) {
     double u1 = randuniform();
     double u2 = randuniform();
     double v1 = 2 * u1 - 1;
     double v2 = 2 * u2 - 1;
     double w = (v1 * v1) + (v2 * v2);
      
/*
 *  We actually generate two IID normal distribution variables.
 *  We cache the one & return the other.
 */ 
     if (w <= 1) {
        double y = sqrt( (-2 * log(w)) / w);
        double x1 = v1 * y;
        double x2 = v2 * y;
     
        haveCachedNormal = 1;
        cachedNormal = x2;
        return(x1 * pStdDev + pMean);
     }
      }
   }

}

MyModule::MyModule(DAQpp::DAQid id, int sz)
    : Module(id)
{
    add_par("pedestals", new DAQpp::Par< std::vector<double> >() );
    add_par("capacity", new DAQpp::Par<int>(sz));
    add_par("eff", new DAQpp::Par<double>(1.));
               
    std::vector<double> &v = get_par_value< std::vector<double> >("pedestals");
    
    int i=0;
    for (i=0;i<10;i++)
        v.push_back(double(i));
}

void MyModule::set_eff(double x, bool notify)
{
    set_par("eff", x, notify);
}
void MyModule::set_capacity(int x, bool notify)
{
    set_par("capacity", x, notify);
}

double MyModule::get_eff()
{
    return get_par_value<double>("eff");
}

int MyModule::get_capacity()
{
    return get_par_value<int>("capacity");
}

int MyModule::PrepareForRun ()
{
    std::cout << "\t+-> Module.PrepareForRun [" 
              << get_id() << "]"
              << std::endl;
    return 0;
}

int MyModule::Start (int mode)
{
    std::cout << "\t+-> Module.Start         [" << get_id() << "]"
    << std::endl;
    return 0;
}

int MyModule::Stop (int mode)
{
    std::cout << "\t+-> Module.Stop          [" << get_id() << "]"
    << std::endl;
    return 0;
}

int MyModule::Pause ()
{
    std::cout << "\t+-> Module.Pause         [" << get_id() << "]"
    << std::endl;
    return 0;
};
int MyModule::Continue ()
{
    std::cout << "\t+-> Module.Continue      [" << get_id() << "]"
    << std::endl;
    return 0;
};
int MyModule::Reset ()
{
    std::cout << "\t+-> Module.Reset         [" << get_id() << "]"
    << std::endl;
    return 0;
};
int MyModule::GetReady ()
{
    std::cout << "\t+-> Module.GetReady      [" << get_id() << "]"
    << std::endl;
    return 0;
};



bool MyModule::hasData()
{
    return (  randuniform() < get_eff() ) ;
}

char *MyModule::getData(int &size)
{
    unsigned long *data;
    char *buff;
    int capacity = get_capacity();

    int chan = int(capacity * randuniform());
    int id = 0x100;
    int i, j;
    double vv;

    int nwords = 4 + (capacity & 0xfff);
    
    size    = nwords * sizeof(long);
    buff    = allocate( size );
    data    = reinterpret_cast<unsigned long *>(buff);
    data[0] = 0xcafe0000 | nwords;
    data[1] = 1;
    data[2] = (id & 0xfff) << 20 | (capacity & 0xfff) | 2 << 12 ;
    for (j = 3, i = 0;i < capacity;i++, j++)
    {
        vv = randgauss(0., 3.);
        data[j] = 200 + ((int)(vv) & 0x7ff);
    }
    data[chan+3] += (int)(randgauss(1000., 50));
    data[j] =  nwords;
    return buff;
}

extern "C"
{
    int create_module(const char *name)
    {
        std::cout << "Creating module "<< name << std::endl;
        DAQpp::Module *m = DAQpp::DAQmanager::theDAQmanager()->find_module(name);
        if (m)
            return 0;

        m=new MyModule(name);
        return 0;
    }
}

