#include <Python.h>
#include <iostream>
#include <cmath>
#include <sys/select.h>
#include <DAQ++/DAQmanager.h>
#include "PythonLoader.h"

PythonLoader PythonLoader::__the_loader;

PythonLoader::PythonLoader() :
    _save(0)
{
    std::cout << "Initializing Python" << std::endl;
    Py_Initialize();

    // Get prepared for the threading...
    if (! PyEval_ThreadsInitialized() )
    {
        PyThreadState *py_tstate = NULL;
        PyEval_InitThreads();
        py_tstate = PyGILState_GetThisThreadState();
        PyEval_ReleaseThread(py_tstate);
    }
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    PyRun_SimpleString("import sys\n"
                           "sys.path.insert(0,'/home/lacasta/projects/daqpp/python/test')\n"
                           );
    PyGILState_Release(gilstate);
}

PythonLoader::~PythonLoader()
{
    std::cout << "Finalizing Python" << std::endl;
    
    ObjectList::iterator ip;
    for (ip=MethodList.begin();ip!=MethodList.end();ip++)
            Py_DECREF(ip->second);
    
    for (ip=ModuleList.begin();ip!=ModuleList.end();ip++)
        Py_DECREF(ip->second);
    
    PyGILState_Ensure();
    Py_Finalize();
}

PythonLoader &PythonLoader::getInstance()
{
    return __the_loader;
}

PyObject *PythonLoader::load_module(const std::string &module, const std::string &method)
{
    PyObject *pymodule, *pymethod;

    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();

    // Load the module
    ObjectList::iterator ip = ModuleList.find(module);
    if (ip==ModuleList.end())
    {
        pymodule = PyImport_ImportModule(module.c_str());
        if (pymodule==0)
        {
            std::cout << "Could not load " << module << std::endl;
            PyGILState_Release(gilstate);
            return 0;
        }
        ModuleList[module] = pymodule;
        Py_INCREF(pymodule);
    }
    else
    {
        pymodule = ip->second;
        Py_INCREF(pymodule);
    }
    PyGILState_Release(gilstate);
    /*
     * Now look for the hook
     */

    std::string name = method + '@' + module;
    ip = MethodList.find(name);
    if (ip==MethodList.end())
    {
        gilstate = PyGILState_Ensure();
        pymethod = PyObject_GetAttrString(pymodule, method.c_str());
        if (!pymethod)
        {
            std::cout << "Could not find " << method << " in " << module << std::endl;
            PyGILState_Release(gilstate);
            return 0;
        }

        if ( !PyCallable_Check(pymethod)) 
        {
            std::cout << method << " in " << module << " is not a callable function" << std::endl;
            PyGILState_Release(gilstate);
            return 0;
        }
        MethodList[name] = pymethod;
        Py_INCREF(pymethod);
    }
    else
        pymethod = ip->second;
    
    PyGILState_Release(gilstate);
    return pymethod;
}

DAQpp::Module *PythonLoader::create_module(const DAQpp::DAQid &id, 
                                           const std::string &module, 
                                           const std::string &method)
{
    PyObject *rc;
    PyObject *pyfunc = load_module(module, method);
    if (!pyfunc)
        return 0;

    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    // Create the module
    rc = PyObject_CallFunction(pyfunc,"s",(const char *)id);
    PyGILState_Release(gilstate);

    if (!rc)
    {
        std::cout << "I could not call " << method << " in " << module << std::endl;
        return 0;
    }
    DAQpp::Module *daq_module = DAQpp::DAQmanager::theDAQmanager()->find_module(id);
    if (!daq_module)
    {
        std::cout << "Something failed when creating module " << id << std::endl;
        return 0;
    }

    return daq_module;
}

DAQpp::RunManager *PythonLoader::create_run_manager(const DAQpp::DAQid &id, 
                                                    const std::string &module, 
                                                    const std::string &method)
{
    PyObject *rc;
    PyObject *pyfunc = load_module(module, method);
    if (!pyfunc)
        return 0;

    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    // Create the module
    rc = PyObject_CallFunction(pyfunc,"s",(const char *)id);
    PyGILState_Release(gilstate);

    if (!rc)
    {
        std::cout << "I could not call " << method << " in " << module << std::endl;
        return 0;
    }
    DAQpp::RunManager *daq_run_manager = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!daq_run_manager)
    {
        std::cout << "Something failed when creating module " << id << std::endl;
        return 0;
    }

    return daq_run_manager;
}

//void PythonLoader::allow_threads()
//{
//    PyThreadState *tstate = PyThreadState_GET();
//    if (tstate==0)
//    {
//        PyGILState_STATE gs = PyGILState_Ensure();
//        gstate.push( gs );
//    }
//}
//
//void PythonLoader::disable_threads()
//{
//    if ( !gstate.empty())
//    {
//        PyGILState_STATE gs = gstate.top();
//        gstate.pop();
//        PyGILState_Release(gs);
//    }
//}
//void PythonLoader::wait(double secs)
//{
//    struct timeval t;
//    double frac;
//    frac = fmod(secs, 1.0);
//    secs = floor(secs);
//    t.tv_sec = (long)secs;
//    t.tv_usec = (long)(frac*1000000.0);
//    Py_BEGIN_ALLOW_THREADS
//    if (select(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, &t) != 0) {
//        if (errno!=EINTR)
//            std::cout << "Error in python_wait: " << strerror(errno) << std::endl;
//    }
//    Py_END_ALLOW_THREADS
//}


