#!/usr/bin/env python

import os, gtk,gobject
import daqpp

class SpinSlider(gtk.HBox):
    """ A SpinSlider is a GtkWidget with a slider and a spin sharing
    the same adjustment so that they are both synchronized.

    It defines it own value_changed signal that will be emited any
    time the adjustment changes its value.
    """
    __gsignals__ = {
        'value_changed' : (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,()),
        }
    def __init__(self, adj):
        super(SpinSlider,self).__init__(False,1)
        if not isinstance(gadj,tk.Adjustment):
            print "SpinSlider should be initialized with an Adjustment"
            print "Creating a dummy one"
            adj = gtk.Adjustment(0.,0., 100., 1., 10., 0.)

        self.create(adj)

    def __init__(self,value, lower, upper, step, page,page_size):
        super(SpinSlider,self).__init__(False,1)
        adj = gtk.Adjustment(value, lower, upper, step, page,page_size)
        self.create(adj)

    def create(self, adj):
        self.adj   = adj
        self.adj.connect("value_changed",self.new_value)

        self.scale = gtk.HScale(self.adj)
        self.spin  = gtk.SpinButton(self.adj,0,1);

        self.pack_start(self.spin,False,True)
        self.pack_start(self.scale,True,True)

        self.scale.set_draw_value(0)
        self.spin.set_snap_to_ticks(True)
        self.spin.set_numeric(True)


    def new_value(self,val):
        self.emit("value_changed")

    def get_value(self):
        return self.adj.get_value()

    def set_value(self,x):
        self.adj.set_value(x)
        
gobject.type_register(SpinSlider)

class RunMenu(gtk.Button):
    """ This is a button that shows and changes the status of
        a RunManager
    """
    __gsignals__ = {
        "command" : (gobject.SIGNAL_RUN_FIRST,
                     gobject.TYPE_NONE,(gobject.TYPE_INT,)),
        }

    notready = -1
    getready =  0
    run      =  1
    stop     =  2
    pause    =  3
    resume   =  4
    reset    =  5

    def __init__(self):
        super(RunMenu,self).__init__("Not Ready")
        self.connect('clicked',self.clicked)

        self.runmenu = self.create_menu()
        self.connect('command',self.command)

        self.queue = []
        self.tmout = gobject.idle_add(self.check_queue)

        self.queue.append(daqpp.NotReady)
        
    def create_menu(self):
        menu = gtk.Menu()
        
        self.igetready    = gtk.MenuItem('Get Ready')
        self.istart       = gtk.MenuItem('Start')
        self.istop        = gtk.MenuItem('Stop')
        self.ipause       = gtk.MenuItem('Pause')
        self.icontinue    = gtk.MenuItem('Continue')
        self.ireset       = gtk.MenuItem('Reset')
        self.separator    = gtk.MenuItem()
        self.quit         = gtk.MenuItem('Quit')

        menu.add(self.igetready)
        menu.add(self.istart)
        menu.add(self.istop)
        menu.add(self.ipause)
        menu.add(self.icontinue)
        menu.add(self.ireset)
        menu.add(self.separator)
        menu.add(self.quit)

        self.igetready.show()
        self.istart.show()
        self.istop.show()
        self.ipause.show()
        self.icontinue.show()
        self.ireset.show()
        self.separator.show()
        self.quit.show()
        self.show_all()

        # We connect to the menu item 'activate' signal
        self.igetready.connect('activate',self.activate,self.getready)
        self.istart.connect('activate',self.activate,self.run)
        self.istop.connect('activate',self.activate,self.stop)
        self.icontinue.connect('activate',self.activate,self.resume)
        self.ipause.connect('activate',self.activate,self.pause)
        self.ireset.connect('activate',self.activate,self.reset)
        self.quit.connect('activate',self.main_quit)

        return menu

    def main_quit(self,w):
        gtk.main_quit()

    def clicked(self,ptr):
        self.runmenu.popup(None,None,None,0,0)
        
    def activate(self, p, command):
        """ We end up here whenever we choose an item from the
            popup menu. p is the menu item and command the command
            chosen
        """
        if command==self.notready or command==self.reset:
            self.igetready.set_sensitive(True)
            self.istart.set_sensitive(False)
            self.istop.set_sensitive(False)
            self.ipause.set_sensitive(False)
            self.icontinue.set_sensitive(False)
            self.ireset.set_sensitive(True)
            
        elif command==self.getready:
            self.igetready.set_sensitive(True)
            self.istart.set_sensitive(True)
            self.istop.set_sensitive(False)
            self.ipause.set_sensitive(False)
            self.icontinue.set_sensitive(False)
            self.ireset.set_sensitive(True)

        elif command==self.run or command==self.resume:
            self.igetready.set_sensitive(False)
            self.istart.set_sensitive(False)
            self.istop.set_sensitive(True)
            self.ipause.set_sensitive(True)
            self.icontinue.set_sensitive(False)
            self.ireset.set_sensitive(True)

        elif command==self.stop:
            self.igetready.set_sensitive(True)
            self.istart.set_sensitive(True)
            self.istop.set_sensitive(False)
            self.ipause.set_sensitive(False)
            self.icontinue.set_sensitive(False)
            self.ireset.set_sensitive(True)

        elif command==self.pause:
            self.igetready.set_sensitive(False)
            self.istart.set_sensitive(False)
            self.istop.set_sensitive(True)
            self.ipause.set_sensitive(False)
            self.icontinue.set_sensitive(True)
            self.ireset.set_sensitive(True)

        # Now we emit the 'command' signal
        if p:
            self.emit('command',command)


    def command(self,w,command):
        pass #print type(w), command
        

    def check_queue(self):
        while len(self.queue)>0:
            state = self.queue.pop(0)
            self.update_state(state)

        return True

    def update_state(self,state):
        lbl = self.child
        if state==daqpp.NotReady or state==daqpp.NotDefined:
            lbl.set_text('Not Ready')
            self.activate(None,self.notready)
        elif state==daqpp.Ready or state==daqpp.GettingReady:
            lbl.set_text('Ready')
            self.activate(None,self.getready)
        elif state==daqpp.Paused:
            lbl.set_text('Paused')
            self.activate(None,self.pause)
        elif state==daqpp.Starting:
            lbl.set_text('Starting')
        elif state==daqpp.Running:
            lbl.set_text('Running')
            self.activate(None,self.run)
        elif state==daqpp.Stopping:
            lbl.set_text('Stopping')
        elif state==daqpp.Stopped:
            lbl.set_text('Ready')
            self.activate(None,self.getready)
        elif state==daqpp.TimeOut:
            lbl.set_text('TimeOut')
            self.activate(None.self.notready)
        else:
            print 'RunManager.update_state: Unhandled state',state
        

    def set_state(self,state):
        self.queue.append(state)

            
gobject.type_register(RunMenu)
