#!/usr/bin/env python
""" This python script shows a simple example of a DAQ++
    session. It defines a RunManager and a Module and runs the
    acquisition system in Threaded mode.
"""
import sys

import daqpp
from simpleModule import MyModule
from simpleRunManager import Rmanager
from optparse import OptionParser


def main():
    parser = OptionParser()
    parser.add_option("--loop",dest="loop",action="store_true",
                      help="Run DAQ++ in Loop mode",
                      default=False)
    parser.add_option("--thread", dest="thread",action="store_true",
                      help="Run DAQ++ in threaded mode",
                      default=True)
    
    (options, args) = parser.parse_args()
    
    rm   = Rmanager("main");
    rm.addModule( MyModule("m.1",512) )
    rm.set_max_events(0)
    m = rm.get_monitor();
    
    try:
        sec = args[0]
    except IndexError:
        sec = 5.
        
    print "Starting run for",sec,"seconds"
    gt = daqpp.GTimer()
    if options.loop:
        print 'Running DAQ++ in Loop mode'
        daqpp.StartRun(daqpp.Loop)
        daqpp.startRunManager(rm)
        rm.Activate()
    else:
        print 'Running DAQ++ in Threaded mode'
        rm.Activate()
        daqpp.StartRun(daqpp.Threaded)
    
    gt.start()
    try:
        while gt()<sec:
            gt.wait(0.25)
            if not m:
                print '\rno-mon: %s' % rm.GetStatus(),
                sys.stdout.flush()
            
    except KeyboardInterrupt:
        print "\nRun interrupted"        

    print '\n\nStopping the run'
    if rm.GetStatus().isRunning():
        daqpp.stopRunManager(rm)
        print '...waiting for RM to finish'
        daqpp.WaitForEndOfRun(100000)
        
    daqpp.StopRun()
    print '...done\n\n'
    print '*** Stats'
    print rm.GetStatus()
        
    print "done"
    
    
if __name__ == "__main__":
    main()
    
