#!/usr/bin/env python
""" This python script shows a simple example of a DAQ++
    session. It defines a RunManager and a Module and runs the
    acquisition system in Threaded mode.
"""
import sys

import daqpp
from simpleModule import MyModule
from simpleRunManager import Rmanager



def main(sec):
    rm   = Rmanager("main");
    rm.addModule( MyModule("m.1",512) )
    rm.set_max_events(0)
    m = rm.get_monitor();
    
    print "Starting run for",sec,"seconds"
    daqpp.StartRun(daqpp.Loop)
    gt = daqpp.GTimer()
    daqpp.startRunManager(rm)
    rm.Activate()
    gt.start()
    try:
        while gt()<sec:
            gt.wait(0.1)
            if not m:
                print '\rno-mon: %s' % rm.GetStatus(),
                sys.stdout.flush()
            
    except KeyboardInterrupt:
        print "\nRun interrupted"        

    print '\n\nStopping the run'
    if rm.GetStatus().isRunning():
        daqpp.stopRunManager(rm)
        print '...waiting for RM to finish'
        daqpp.WaitForEndOfRun(100000)
        
    daqpp.StopRun()
    print '...done\n\n'
    print '*** Stats'
    print rm.GetStatus()
        
    print "done"
    
    
if __name__ == "__main__":
    try:
        sec = float(sys.argv[1])
    except IndexError:
        sec = 5.
    main(sec)
    
