#!/usr/bin/env python
"""
soapClient

 Created on: Jan 10, 2011
     Author: lacasta
"""
import os, sys
import time
import struct
import base64
import SOAPpy
import daqpp.soap
import Histogram
from SOAPpy import SOAP

def main(host, port):
    server = SOAPpy.SOAPProxy("http://%s:%d/daqpp" %(host, port), namespace=daqpp.soap.DAQPP_NS, noroot=1)

    update = server.sendUpdate(0)
    print daqpp.soap.parse_update(update)

    modules = []
    ml = server.getModuleList(0)
    if isinstance(ml.name, str):
        ml.name = (ml.name,)
        
    for name in ml.name:
        dobj = daqpp.soap.DAQObject(name, server)
        dobj.resetMonitorData()
        modules.append(dobj)
        
    

    rml = server.getRunManagerList(0)
    # When there is only one answer, SOAPpy sends a string, not a list of strings
    if isinstance(rml.name, str):
        rml.name = (rml.name,)

    for name in rml.name:
        print "RunManager: ", name
        for m in server.getRunManagerModules(daqid=name).name:
            print "\tModule:", m

    for module in modules:
        print "Module", module.name
        for pname, par in module.parameters.items():
            print "\tPar. ", par.name, " type ", par.type  #, "value: ", par.get_value()

    run_manager = daqpp.soap.RunManager("main", server)
    status = run_manager.getStatus()

    run_manager.setMaxEvents(-1)
    run_manager.GetReady()
    run_manager.startRun()
    t0 = time.time()
    while (time.time() - t0 < 2.5):
        time.sleep(0.1)
        print '\r', run_manager.getStatus(),
        sys.stdout.flush()

    run_manager.stopRun()
    print ''

    #modules[0].parameters['mean'].set_value(10.0)
    new_val = [2.0*x + 1.0  for x in range(0, 10)]
    modules[1].parameters['pedestals'].set_value( new_val )
    val = modules[1].parameters['pedestals'].get_value()
    print val

    for m in modules:
        if m.name == 'm.X':
            m.parameters['pedestals'].set_value( [2.0*x + 1.0  for x in range(0, 10)])
            val = m.parameters['pedestals'].get_value()
            print val


    update = server.sendUpdate(0)
    st_update, par_update = daqpp.soap.parse_update(update)
    print "*** last update:"
    print "  - States"
    for st in st_update:
        print '\t', st

    print "  - Params"
    for par in par_update:
        print '\t', par

    print "*** Getting monitor data ***"
    mdata = modules[0].getMonitorData() #server.getMonitorData(daqid=modules[0].name)
    val = base64.decodestring(mdata)
    
    hst = Histogram.Histogram("hst", "The Histogram", 1, 0.0, 1.0)
    hst.read(val)
    hst.dump()


if __name__ == "__main__":
    main("localhost", 50000)
