#!/usr/bin/env python
import sys

import os, time
import gtk
import array
import gtkhst,gobject
import daqpp
import random
from gtktools import SpinSlider, RunMenu
from MsgManager import MsgManager

class MyTrigger(object):
    def __init__(self,freq):
        self.freq=freq
    
    def __call__(self):
        return (random.random() < self.freq)
    
class MyRunMenu(RunMenu):
    """ This is a variation of a RunMenu that sends commands
        to the RunManager
    """
    def __init__(self, runm):
        super(MyRunMenu,self).__init__()
        
        # We become Observers of the RunManager state
        self.runm = runm
        self.subscribe(runm)
        self.cmd_queue=[]
        
        # We connect to the 'command' signal to add it to the queue
        self.connect('command', self.append_command)
        
        # Create an idle function to check for new commands
        gobject.idle_add(self.check_commands)

    def subscribe(self,runm):
        """ Add ourselves as Observers of the RunManager state
        """
        self.runm = runm
        if runm:
            runm.addObserver( state=self )
            
    def update(self,arg):
        """ The call back method that is called when the RunManagers
            changes its state
        """
        # This will trigger the 'command' signal of the RunMenu
        self.set_state(arg)


    def check_commands(self):
        """ Checks whether there are new commands and 'execute'them
        """
        if len(self.cmd_queue):
            cmd = self.cmd_queue.pop(0)
            self.do_command(cmd)

    def append_command(self, w, command):
        """ Called when the RunMenu emits the command signal 
        """
        self.cmd_queue.append(command)

    def do_command(self, command):
        """ Execute the command
        """
        if not self.runm:
            return

        if command==self.notready or command==self.reset:
            if self.runm.isActive():
                self.runm.deActivate()

        elif command==self.getready:
            self.runm.GetReady()

        elif command==self.run:
            if not self.runm.isActive():
                self.runm.Activate()

            print "RunManager",self.runm.get_id(),": active [",self.runm.isActive(),"]"
            daqpp.startRunManager( self.runm )

        elif command==self.stop:
            daqpp.stopRunManager( self.runm )
            print self.runm.GetStatus()
            for m in self.runm.get_modules():
                print m, daqpp.find_module(m).GetStatus()

        elif command==self.pause:
            self.runm.Pause()

        elif command==self.resume:
            self.runm.Continue()

        else:
            print 'MyRunMenu.command: Unhandled command',command




    

class MyMonitor(daqpp.Monitor):
    def __init__(self):
        super(MyMonitor,self).__init__()
        self.hst = None
        self.keep_monitoring = True
        self.nevts = 0

    def on_start(self):
        self.nevts = 0
        
    def on_stop(self):
        print "*** Monitor: %d events processed" % self.nevts

    def set_hst(self,hst):
        self.hst = hst


    def idle(self, *args):
        rm = self.get_runmanager()
        if rm:
            print ('\r%s' % rm.GetStatus()),
            
        buf = self.get_event()
        if buf:
            buf.type = 'd'
            self.hst.fill(buf[0], 1)
            del(buf)

        return self.keep_monitoring


    def run(self,buffer):
        if self.hst and buffer:
            self.nevts += 1
            buffer.type='d'
            self.hst.fill( buffer[0], 1)
        #print '\r%s [%d]' % (self.get_runmanager().GetStatus(), self.nevts), 
        return

class Rmanager(daqpp.RunManager):
    def __init__(self,id):
        super(Rmanager,self).__init__(id)
        monitor = MyMonitor()
        self.set_monitor(monitor)
        self.cntr = 0        
        self.add_par("slider", 0.0)

    def GetReady(self):
        print self.get_id(),"-> GetReady"
        super(Rmanager,self).GetReady()

    def PrepareForRun(self):
        print self.get_id(),"-> PrepareForRun"
        super(Rmanager,self).PrepareForRun()
        monitor = self.get_monitor()
        if monitor:
            monitor.hst.reset()

    def writeData(self,buff):
        self.cntr += 1
        return buff.size

    def set_slider_value(self, spl):
        self.set_par("slider", spl.get_value(), True)


class MyModule(daqpp.Module):
    def __init__(self,id,eff=0.5,size=256,mean=0.,sigma=1.):
        super(MyModule,self).__init__(id)
        self.eff = eff
        self.mean = mean
        self.sigma = sigma
        self.cntr = 0
        self.array =  array.array('d',[0 for x in range(0,size)])

    def foo(self):
        return id(self)

    def PrepareForRun(self):
        print self.get_id(),"\t+-> PrepareForRun"
        return 1

    def Start(self,mode):
        print self.get_id(),"\t+-> Start"
        return 1

    def Stop(self,mode):
        print self.get_id(),"\t+-> Stop"
        return 1

    def hasData(self):
        xx = (random.random() < self.eff)
        return xx

    def getData(self):
        self.array[0] = random.gauss(self.mean,self.sigma)
        self.cntr += 1
        self.cntr = (self.cntr % 100)
        return self.array




class MainWindow(dict):
    def __init__(self):
        self.create()

    def __getattr__(self, data_name):
        if data_name in self:
            data = self[data_name]
            return data
        else:
            widget = self.window.get_data(data_name)
            if widget != None:
                self[data_name] = widget
                return widget
            else:
                raise AttributeError, data_name
                        
    def __setattr__(self, name, value):
        self[name] = value
            
    def new_value(self,spl):
        spl.get_value()
            
    def create(self):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title("testGtk");
        self.window.connect('delete_event',self.gtk_true)

        vbox = gtk.VBox(False,1)
        self.window.add(vbox)
        
        hbox = gtk.HBox(False,1)
        vbox.pack_start(hbox,False,True)


        #
        # Create the spin slider
        #
        vControl = gtk.VBox(False,1)
        hbox.pack_start(vControl,False,True)
        
        self.spl = SpinSlider(0.,0., 100., 1., 10., 0.)
        vControl.pack_start(self.spl,False,True)

        self.spl.connect("value_changed",self.new_value)
        self.spl.set_size_request(200,-1)

        #
        # The run manager
        #
        self.runm = MyRunMenu(None)
        vControl.pack_start(self.runm,False,True)

        #
        # The Histogram
        #
        self.hst = gtkhst.Histogram(50,-25.5,25.5)
        hst = gtkhst.Hst1D( self.hst )
        hst.monitor_start(100)
        hst.filled(True)
        vControl.pack_start( hst, False, True )

        #
        # The gauges
        #
        vMonitor = gtk.VBox(False,1)
        hbox.pack_start(vMonitor,False,True)

        
        self.rate = gtkhst.Gauge( 0.,50000. )
        self.rate.set_title("Rate")
        self.rate.set_color("green")
        vMonitor.pack_start(self.rate,False,True)

        self.eff = gtkhst.Gauge( 0.,100. )
        self.eff.set_title("Efficiency")
        self.eff.set_color("blue")
        vMonitor.pack_start(self.eff,False,True)


        #
        # The throughput monitor
        #
        self.throughput = gtkhst.Tracer(51)
        tracer =  gtkhst.Hst1D(  self.throughput )
        tracer.set_colors(foreground="red",background="black",axis="green");
        tracer.monitor_start(150)
        tracer.draw_x_axis(gtkhst.AXIS_NONE)
        tracer.draw_y_axis(gtkhst.AXIS_MIDDLE)
        vMonitor.pack_start(tracer,False,True)

        #
        # The button box
        #
        btnbox = gtk.HButtonBox()
        btnbox.show()
        vbox.pack_start(btnbox,False,True)
        
        btn = gtk.Button(stock=gtk.STOCK_OK)
        btn.connect('clicked',self.gtk_main_quit)
        
        btnbox.pack_start(btn)
        
        
        self.window.show_all()
        

    def update(self,data):
        rm = daqpp.find_runmanager("main")
        if not rm:
            return 1

        st = rm.GetStatus()
        self.rate.set_value( st.rate(False) )
        self.eff.set_value( st.eff() )
        self.throughput.fill( st.throughput()/1024./1024./st.time(),1. )

        return 1

    def gtk_true(self, *args):
        return True
	
    def gtk_false(self, *args):
        return False
	
    def gtk_main_quit(self, *args):
        gtk.main_quit()
		
    def main(self):
        gtk.main()

    def quit(self):
        gtk.main_quit()

    def run(self):
        try:
            gtk.gdk.threads_init()
            gobject.timeout_add(100,self.update,None)
            self.main()
        except KeyboardInterrupt:
            self.on_keyboard_interrupt()


    def on_keyboard_interrupt(self):
         pass       

def main():
    #
    # Add the MsgManager
    #
    daqpp.add_observer(MsgManager())
    
    rm   = Rmanager("main");
    rm.addModule( MyModule("m.1",0.75,512,-10) )
    rm.addModule( MyModule("m.2",1,128,10,2) )
    rm.set_max_events(0)

    daqpp.set_trigger( MyTrigger(0.1) )
    
    window = MainWindow()
    window.runm.subscribe(rm)
    window.spl.connect("value_changed", rm.set_slider_value)

    daqpp.StartRun(daqpp.Loop)


    monitor = rm.get_monitor()
    if monitor:
        monitor.timeout = 100
        monitor.set_hst( window.hst )
    
        #gobject.timeout_add(100,monitor,rm)
    window.run()
    print rm.GetStatus()

if __name__ == "__main__":
    main()


#-------------------------------------------------------------------------
