#!/usr/bin/env python
import sys
import random
import array
import time
import daqpp
from simpleMonitor import MyMonitor

class MyTrigger(object):
    def __init__(self,freq):
        self.freq=freq
        print "new Trigger",self.freq

    def __call__(self):
        xx = (random.random() < self.freq)
        return xx



class Rmanager(daqpp.RunManager):
    def __init__(self,id):
        super(Rmanager,self).__init__(id)
        monitor = MyMonitor()
        monitor.timeout = 100;
        self.set_monitor(monitor)
        self.cntr = 0

    def GetReady(self):
        print self.get_id(),"-> GetReady"
        super(Rmanager,self).GetReady()

    def Start(self):
        print self.get_id,"-> Start"
        return super(Rmanager, self).Stop()

    def Stop(self):
        print self.get_id(),"-> Stop"

        return super(Rmanager,self).Stop()

    def PrepareForRun(self):
        print self.get_id(),"-> PrepareForRun"
        super(Rmanager,self).PrepareForRun()
        

    def EndOfRun(self):
        print self.get_id(),"-> EndOfRun"
        super(Rmanager,self).EndOfRun()
        
    def writeData(self, buff):
        return buff.size



class MyModule(daqpp.Module):
    def __init__(self,id,eff=0.5,size=256):
        super(MyModule,self).__init__(id)
        self.eff = eff
        self.cntr = 0
        self.array =  daqpp.Buffer('d',size)
        self.array.id = self.get_id()

    def foo(self):
        return id(self)

    def PrepareForRun(self):
        print self.get_id(),"\t+-> PrepareForRun"
        return 1

    def Start(self,mode):
        print self.get_id(),"\t+-> Start",mode
        return 1

    def Stop(self,mode):
        print self.get_id(),"\t+-> Stop",mode
        return 1

    def hasData(self):
        xx = (random.random() < self.eff)
        return xx

    def getData(self):
        self.array[0] = self.cntr
        self.cntr += 1
        self.cntr = (self.cntr % 100)
        return self.array

def setup_daq():
    rm   = Rmanager("main");

    theTrigger = MyTrigger(1)
    daqpp.set_trigger(theTrigger)

    rm.addModule( MyModule("m.1",0.75,512) )
    rm.addModule( MyModule("m.2",0.5,128) )
    rm.addModule( MyModule("m.3",0.25) )
    

    print "Number of modules in ",rm.get_id(),"is",rm.countModules()
    print "Number of modules in DAQmanager",daqpp.countModules()
    print "Done with setup"

def testThreaded():
    print '####'
    print '#### Testing Threaded mode'
    print '####'
    
    rm = daqpp.find_runmanager("main");

    rm.Activate()
    rm.set_max_events(0)
    m = rm.get_monitor();
    
    print "Starting run"
    daqpp.StartRun(daqpp.Threaded)
    gt = daqpp.GTimer()
    gt.start()
    try:
        while gt()<2.5:
            time.sleep(0.1)
            if not m:
                print '\rno-mon: %s' % rm.GetStatus(),
                sys.stdout.flush()
            
    except KeyboardInterrupt:
        print "\nRun interrupted"        

    print '\n\nStopping the run\n\n'
    if rm.GetStatus().isRunning():
        daqpp.stopRunManager(rm)
        print '...waiting for RM to finish'
        daqpp.WaitForEndOfRun(100000)
        
    daqpp.StopRun()
    print '\n\n...done\n\n'
    print rm.GetStatus()
        
    print "done"


def testLoop():
    #
    #  Now we demonstrate the Loop way of running.
    #  First we deactivate the RunManager and we start the run.
    #  In principle the DAQmanaer loop will be waiting for active
    #  RunManager objects
    #
    print '####'
    print '#### Testing Loop mode'
    print '####'
    rm = daqpp.find_runmanager("main");
    mon = rm.get_monitor()
    
    if rm.isActive():
        print "Deactivating",rm.get_id()
        rm.deActivate()


    # Now activate the DAQmanager in the background
    daqpp.StartRun(daqpp.Loop)

    for i in range(0,5):
        ss = "m.%d" % ((i%3)+1)
        m = daqpp.find_module(ss)
        if m==None:
            print "\n\n##### Module ",i,"does not exist"
            continue
        
        print "\n\n##### ",i,"module:",m.get_id(),"in local"
        m.in_local()

        rm.Activate()
        rm.set_max_events(100000)
        daqpp.startRunManager(rm)
        gt = daqpp.GTimer()
        gt.start()
        try:
            while rm.GetStatus().isRunning() and gt()<2.5:
                time.sleep(0.1)
                if not mon:
                    print '\rno-mon: %s' % rm.GetStatus(),
                    sys.stdout.flush()
        
        except KeyboardInterrupt:
            print "\nRun interrupted"
            break
            
        if rm.GetStatus().isRunning():
            daqpp.stopRunManager(rm)
            print '...waiting for RM to finish'
            daqpp.WaitForEndOfRun(100000)
        
        m.in_global()
        print m.get_id()," in global ",m.is_global()
        print rm.GetStatus()
        
    daqpp.StopRun()
    print "done"

def main():
    setup_daq()
    testThreaded()
    testLoop()

if __name__ == "__main__":
    main()
#--------------------------------------------------------------------
