#!/usr/bin/env python
""" Here we test how to load a C++ module
"""
import sys
import random
import array
import time
import daqpp

from simpleRunManager import Rmanager

class ParObserver(object):
    def __init__(self, par, name):
        self.par = par
        self.par.addObserver(self)

    def update(self, val):
        print '### parameter',self.par.name,' has changed to:',val
    
def create_module(name, hook):
    hook.create_module(name)
    return daqpp.find_module(name)
     
def setup_daq():
    rm   = Rmanager("main");

    #hook = daqpp.load_lib("libmodule.so",daqpp.RTLD_LAZY)

    #theTrigger = MyTrigger(1)
    #daqpp.set_trigger(theTrigger)

    #module = create_module('m.1',hook)
    module = daqpp.create_module('m.1', "create_module", "libmodule.so")
    rm.addModule( module )

    print 'Module',module.get_id(),'has the following parameters'
    for par in module.par_names():
        sp = ParObserver(module.find_par(par), par)
        print '...param: ',par,'value=',module.get_par(par)

    print "Now I will change 'eff' and we should see someone realizing of that..."
    module.set_par('eff',0.75,True)
    
    print "Now I change pedestals"
    module.set_par('pedestals',(2.,4.,6.,8.), True)
    print "...new value",module.get_par('pedestals')

    print "Now I change the capacity"
    module.set_par('capacity',512, True)

    print "Number of modules in ",rm.get_id(),"is",rm.countModules()
    print "Number of modules in DAQmanager",daqpp.countModules()
    print "Done with setup"
    
def main():
    setup_daq()
    rm = daqpp.find_runmanager("main");

    rm.Activate()
    rm.set_max_events(0)
    m = rm.get_monitor();
    
    module = daqpp.find_module("m.1")
    print "module parameters", module.par_names()
    
    print "\n\nStarting run"
    daqpp.StartRun(daqpp.Threaded)
    gt = daqpp.GTimer()
    gt.start()
    try:
        while gt()<2.5:
            time.sleep(0.1)
            if not m:
                print '\rno-mon: %s' % rm.GetStatus(),
                sys.stdout.flush()
            
    except KeyboardInterrupt:
        print "\nRun interrupted"        

    print '\n\nStopping the run\n\n'
    if rm.GetStatus().isRunning():
        daqpp.stopRunManager(rm)
        print '...waiting for RM to finish'
        daqpp.WaitForEndOfRun(100000)
        
    daqpp.StopRun()
    print '\n\n...done\n\n'
    print rm.GetStatus()
        
    print "done"
    
    
if __name__ == "__main__":
    main()
    
