#!/usr/bin/env python
""" Test the trigger facility
"""

import sys
import daqpp
import random
from simpleModule import MyModule
from simpleRunManager import Rmanager
from optparse import OptionParser

class MyTrigger(object):
    """ Our trigger object. 
        This object should be a callable object, so we define the __call__ method
        It could also be a function.
        
    """
    def __init__(self,freq):
        self.freq=freq
        self.period = 1.0/self.freq
        self.gt = daqpp.GTimer()
        
        
    def __call__(self):
        val = random.gauss(self.period, 0.05*self.period)
        self.gt.wait(val)
        return True;
    
def main():
    """ First argument is the desired frequency
    """
    try:
        freq = float(sys.argv[1])
    except IndexError:
        freq = 50.
        
    my_trigger = MyTrigger(freq)
    daqpp.set_trigger(my_trigger)
    
    rm   = Rmanager("main");
    rm.addModule( MyModule("m.1",512) )
    rm.set_max_events(0)
    m = rm.get_monitor();
    
    
    rm.Activate()
    daqpp.StartRun(daqpp.Threaded)
    
    gt = daqpp.GTimer()
    try:
        while gt()<10.0:
            gt.wait(0.25)
            if not m:
                print '\rno-mon: %s' % rm.GetStatus(),
                sys.stdout.flush()
            
    except KeyboardInterrupt:
        print "\nRun interrupted"        

    print '\n\nStopping the run'
    if rm.GetStatus().isRunning():
        daqpp.stopRunManager(rm)
        print '...waiting for RM to finish'
        daqpp.WaitForEndOfRun(100000)
        
    daqpp.StopRun()
    print '...done\n\n'
    print '*** Stats'
    print rm.GetStatus()
        
    print "done"
    
    
if __name__ == "__main__":
    main()
    