# a macro to get the libs/cflags for libglade
# serial 1

dnl AM_PATH_DAQPP([MINIMUM-VERSION,[ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND [, MODULES]]]])
dnl Test to see if DAQ++ is installed, and define DAQPP_CFLAGS, LIBS
dnl
AC_DEFUN([AM_PATH_DAQPP],
[dnl
dnl Get the cflags and libraries from the DAQ++-config script
dnl
AC_ARG_WITH(DAQ++-config,
[  --with-daqpp-config=DAQPP_CONFIG  Location of DAQ++-config],
DAQPP_CONFIG="$withval")

AC_PATH_PROG(DAQPP_CONFIG, DAQ++-config, no)

min_daqpp_version=ifelse([$1], ,0.0.0,$1)
AC_MSG_CHECKING(for DAQ++ - version >= $min_daqpp_version)



module_args=
for module in . $4; do
  case "$module" in
    gtk)
      module_args="$module_args gtk"
      ;;
  esac
done

if test "$DAQPP_CONFIG" = "no"; then
  AC_MSG_RESULT(no)
  ifelse([$3], , :, [$3])
else
  if $DAQPP_CONFIG --check $module_args; then
    DAQPP_CFLAGS=`$DAQPP_CONFIG --cflags $module_args`
    DAQPP_LIBS=`$DAQPP_CONFIG --libs $module_args`
    test_major=`echo $min_daqpp_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    test_minor=`echo $min_daqpp_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    test_micro=`echo $min_daqpp_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
	
    daqpp_major_version=`$DAQPP_CONFIG --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    daqpp_minor_version=`$DAQPP_CONFIG --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    daqpp_micro_version=`$DAQPP_CONFIG --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    	
    min_version=$((10000*${test_major}+100*${test_minor}+${test_micro}))
    cur_version=$((10000*$daqpp_major_version+100*$daqpp_minor_version+$daqpp_micro_version))

    if eval "test $cur_version -ge $min_version" ; then
	AC_MSG_RESULT(yes)
    else
	AC_MSG_RESULT(no)
	ifelse([$3], , :, [$3])
    fi	
    ifelse([$2], , :, [$2])
  else
    echo "*** DAQ++ was not compiled with support for $module_args" 1>&2
    AC_MSG_RESULT(no)
    ifelse([$3], , :, [$3])
  fi
fi
AC_SUBST(DAQPP_CFLAGS)
AC_SUBST(DAQPP_LIBS)
])