#ifndef __DataLogger_h__
#define __DataLogger_h__

#include <DAQ++/FileIOManager.h>
#include <DAQ++/RunManager.h>
#include <DAQ++/Fifo.h>
#include <DAQ++/Thread.h>

/**
 This is an example of the implementation of a data logger. The
 logger works as a fifo into which the RunManager will dump the
 data. It derives from FileIOmanager, and that interface will take
 care of the IO part. It runs a thread waiting for data in the fifo
 and as soon as it is available it will dump it into the file.
 */
class DataLogger: public DAQpp::FileIOManager
{
    private:
        // The fifo that will hold the data.
        DAQpp::Fifo fifo;

        bool stoplogging;

        // This is the thread
        class Logger: public DAQpp::Thread
        {
            private:
                DataLogger *log;
            public:
                Logger(DataLogger *l) :
                    log(l)
                {
                }
                void run()
                {
                    if (log)
                        log->log_data();
                }
        };
        friend class Logger;
        Logger plogger;

        void log_data();
    public:
        DataLogger(DAQpp::RunManager *r, int buffer_size, bool compress = true,
                   unsigned long mx_size = 500000);
        ~DataLogger();

        void start_logging();
        void stop_logging();
        int writeData(int size, const char *data)
        {
            return fifo.write(const_cast<char *> (data), size);
        }
};

#endif
