#ifndef __HISTOS_inc__
#define __HISTOS_inc__
#include <fstream>
#include <cstring>

class Histogram
{
public:
    Histogram (const char *, const char *, int, double, double);
    Histogram (const Histogram &h);
    virtual ~Histogram();

    friend std::ostream& operator<< ( std::ostream& , const Histogram&  );
    friend Histogram &sqrt(const Histogram &);

    Histogram const &operator=(Histogram const &h);
    double &operator[](int);

    int Nbin() const
    {
        return nx;
    };
    const char *Name() const
    {
        return name;
    };
    const char *Title() const
    {
        return title;
    }
    double Step() const
    {
        return step;
    }
    double Xmax() const
    {
        return xmax;
    };
    double Xmin() const
    {
        return xmin;
    };
    double Hsum() const
    {
        return hsum;
    }   // returns contents of histogram
    double Tsum() const
    {
        return tsum;
    }   // returns hsum + overf. + underf.
    double Mean();                  // returns mean value
    double Var();                   // returns variance
    double stdDev();                // returns standar deviation
    double BinCenter(int i);
    int    Bin(double x);
    void   GetData(double *ptr, int size);
    void   SetData(double *ptr, int size);
    virtual void   Fill(double x, double w);
    virtual void   Fill(double x, double y, double w);
    virtual void Print(std::ostream &os = std::cout) const;

protected:
    void init(double val = 0.);
    void copy(Histogram const &h);
    void destroy();
protected:
    int         nx;
    double      xmin, xmax;
    double      hsum, tsum;
    double      step;
    double     *data;
    char       *name;
    char       *title;
};

class HProfile : public Histogram
{
public:
    HProfile(const char *name, const char *title,
             int n, double x1, double x2,
             double y1 = -1.e-30, double y2 = 1.e+30);
    virtual ~HProfile();

    virtual void Fill(double, double w = 1.);
    virtual void Fill(double, double, double w = 1.);
    virtual void Print(std::ostream &os = std::cout) const;
protected:
    double *sum;
    double *err;
    double ymin, ymax;
};


inline double Histogram::BinCenter(int x)
{
    return xmin + (0.5 + nx)*step;
}

inline int Histogram::Bin(double x)
{
    return int( nx*(x - xmin) / (xmax - xmin)  );
}

inline void Histogram::GetData(double *ptr, int size)
{
    memcpy(ptr, data, sizeof(double)*size);
}

inline void Histogram::SetData(double *ptr, int size)
{
    memcpy(data, ptr, size*sizeof(double));
}
#endif
