/*
 * client-param.cc
 *
 *  Created on: 27 Dec 2010
 *      Author: lacasta
 */

#include <iostream>
#include <iomanip>
#include <vector>
#include <algorithm>
#include <iterator>
#include <DAQ++/soap/soapDAQppClient.h>
#include <DAQ++/soap/logging.h>

int main(int argc, char **argv)
{
    int i, rc;
    daqppSOAPProxy client;
    ns1__DAQidList modules;
    ns1__getParameterResponse parameter;
    ns1__ParameterID par_id;
    ns1__Parameter *ppar, *par;

    client.soap_endpoint = "http://localhost:50000";

    double vval[] = {2.,4.,6.,8.,10.};
    std::vector<double> new_pedestals(vval, &vval[5]);



//    soap_register_plugin(&client, logging);
//    soap_set_logging_outbound(&client, stdout);
//    soap_set_logging_inbound(&client, stdout);


    par_id.holderID = "mod.1";
    par_id.parName = "pedestals";
    std::vector<double> pedestals;

    client.getParameter(&par_id, parameter);
    par = parameter.param;
    get_vpar_value<double, xsd__double>(par, pedestals);

    std::cout << "Parameter " << par->value->name << '/' << par->daqid << " of type " << par->value->type << std::endl;
    std::cout << "<";
    std::copy(pedestals.begin(), pedestals.end(), std::ostream_iterator<double>(std::cout, " "));
    std::cout << ">" << std::endl;

    par_id.parName = "random";
    client.getParameter(&par_id, parameter);
    par = parameter.param;
    std::cout << "Parameter " << par->value->name << " of type " << par->value->type
              << " has value " << get_par_value<bool, xsd__boolean>(par) << std::endl;

    par_id.parName = "mean";
    client.getParameter(&par_id, parameter);
    par = parameter.param;
    std::cout << "Parameter " << par->value->name << " of type " << par->value->type
              << " has value " << get_par_value<double, xsd__double>(par) << std::endl;

    par_id.parName = "std";
    client.getParameter(&par_id, parameter);
    par = parameter.param;
    std::cout << "Parameter " << par->value->name << " of type " << par->value->type
              << " has value " << get_par_value<double, xsd__double>(par) << std::endl;

    par_id.parName = "name";
    client.getParameter(&par_id, parameter);
    par = parameter.param;
    std::cout << "Parameter " << par->value->name << " of type " << par->value->type
              << " has value " << get_par_value<std::string, xsd__string>(par) << std::endl;

    // Now we start changing values
    ppar = set_parameter<double, xsd__double>(&client, "mean", 5.);
    ppar->daqid = "mod.1";
    client.setParameter(ppar, rc);

    ppar = set_vparameter<double, xsd__double>(&client, "pedestals", new_pedestals);
    ppar->daqid = "mod.1";
    client.setParameter(ppar, rc);

    // now test the xtreme
    for (i=0; i<15000; i++)
    {
        if ( !(i%1000) )
        {
            std::cout << '\r' << std::setw(5) << i;
            std::cout.flush();
        }
        ppar = set_vparameter<double, xsd__double>(&client, "pedestals", new_pedestals);
        ppar->daqid = "mod.1";
        client.setParameter(ppar, rc);

        par_id.holderID = "mod.1";
        par_id.parName = "pedestals";
        client.getParameter(&par_id, parameter);
    }
    std::cout << std::endl;

    if (argv[1])
        client.stopServer(rc, rc);

    return 0;
}
