/*
 * client.cc
 *
 *  Created on: Dec 21, 2010
 *      Author: lacasta
 */

#include <cstdio>
#include <string>
#include <sstream>
#include <unistd.h>
#include <DAQ++/soap/soapdaqppSOAPProxy.h>
#include <DAQ++/soap/logging.h>
#include <DAQ++/soap/soapDAQppClient.h>
#include <DAQ++/GTimer.h>
#include <DAQ++/logger.h>
#include <DAQ++/soap/utils.h>
#include "Histogram.h"
#include <getopt.h>
#include <DAQ++/soap/daqppClient.h>


int main(int argc, char **argv)
{
    int rc;
    daqppSOAPProxy client;

    int debug_in=0, debug_out=0, stop_server=0;
    int option_index;
    static struct option long_options[] =
    {
     { "debug_in", 0, &debug_in, 1 },
     { "debug_out", 0, &debug_out, 1 },
     { "stop_server", 0, &stop_server, 1},
     { 0, 0, 0, 0 }
    };
    while ( 1)
    {
        char c;
        option_index = 0;
        c = getopt_long(argc, argv, "", long_options, &option_index);
        if (c == -1)
            break;

        switch (c)
        {
            case 0:
                break;
            default:
                std::cout << "Error " << argv[optind-1] << " ";
                if (optind<argc)
                    std::cout << argv[optind];
                std::cout << " [Ughhhh !!!!]" << std::endl;
                return 1;

        }
    }




    soap_register_plugin(&client, logging);
    if (debug_out)
        soap_set_logging_outbound(&client, stdout);

    if (debug_in)
        soap_set_logging_inbound(&client, stdout);

    // send response
    client.soap_endpoint = "http://localhost:50000";

    DAQpp::ostream odaq;

    odaq << DAQpp::loglevel(DAQpp::Log::info) << "Sending an update request " << std::endl;
    ns1__sendUpdateResponse update;
    client.sendUpdate(0, update);


    odaq << DAQpp::loglevel(DAQpp::Log::info) << "Starting a run lasting 5 sec." << std::endl;
    ns1__MaxEvents mxevts;
    mxevts.daqid="main";
    mxevts.nevts= 100000;
    client.setMaxEvents(&mxevts, rc);
    client.GetReady("main", rc);

    DAQpp::GTimer timer;
    timer.start();
    client.StartRunManager("main", rc);
    while (timer()<5.0)
    {
        daqpp_wait(0.05);
    }
    timer.stop();
    odaq << DAQpp::loglevel(DAQpp::Log::info) << "Stopping the run" << std::endl;
    client.StopRunManager("main", rc);
    odaq << DAQpp::loglevel(DAQpp::Log::info) << "Run is stopped. Status is"
         << std::endl;



    // Now we start changing values
    odaq <<  DAQpp::loglevel(DAQpp::Log::info) << "Changing parameters mean in mod.1" << std::endl;
    ns1__Parameter *ppar = set_parameter<double, xsd__double>(&client, "mean", 5.);
    ppar->daqid = "mod.1";
    client.setParameter(ppar, rc);


    odaq <<  DAQpp::loglevel(DAQpp::Log::info) <<  "See whether this is given by the updates" << std::endl;
    std::cout << "**** An update ***" << std::endl;
    client.sendUpdate(0, update);

    std::cout << "*** Updates:" << std::endl;
    if ( ! update.update->status->status.empty() )
    {
        std::cout << "  - Status" << std::endl;
        std::vector<ns1__Status *>::iterator is;
        for (is=update.update->status->status.begin(); is!=update.update->status->status.end(); is++)
        {
            std::cout << '\t' << *(*is) << std::endl;
        }
    }
    if (!update.update->parameters->parameter.empty())
    {
        std::cout << "  - Parameters" << std::endl;
        std::vector<ns1__Parameter *>::iterator ip;
        for (ip=update.update->parameters->parameter.begin(); ip!=update.update->parameters->parameter.end(); ip++)
        {
            ns1__Parameter *p = *ip;
            ns1__ParameterValue *pv = p->value;
            //ns1__valueType *vt = pv->value;
            std::cout << "\tParameter " << pv->name << '@' << p->daqid << " of type " << p->value->type << std::endl;
        }
    }


    if (stop_server)
        client.stopServer(rc, rc);

    return 0;
}

